# -*- coding: utf-8 -*-
"""
Stopwords from the nltk package

Bird, Steven, Edward Loper and Ewan Klein (2009).
Natural Language Processing with Python.  O'Reilly Media Inc.

List of stopwords useful in text mining, analyzing content of tweets, web
pages, keywords, etc.

Each list is accessible as part of a dictionary `stopwords` which is a normal
Python dictionary.

>>> import advertools as adv
>>> adv.stopwords['english'][:5]
['i', 'me', 'my', 'myself', 'we']

>>> adv.stopwords['german'][:5]
['aber', 'alle', 'allem', 'allen', 'aller']

>>> adv.stopwords.keys()
dict_keys(['arabic', 'azerbaijani', 'danish', 'dutch', 'english', 'finnish', 'french', 
'german', 'greek', 'hungarian', 'italian', 'kazakh', 'nepali', 'norwegian', 'portuguese', 
'romanian', 'russian', 'spanish', 'swedish', 'turkish'])
"""


stopwords = {

    'arabic':
    [
        'إذ',
        'إذا',
        'إذما',
        'إذن',
        'أف',
        'أقل',
        'أكثر',
        'ألا',
        'إلا',
        'التي',
        'الذي',
        'الذين',
        'اللاتي',
        'اللائي',
        'اللتان',
        'اللتيا',
        'اللتين',
        'اللذان',
        'اللذين',
        'اللواتي',
        'إلى',
        'إليك',
        'إليكم',
        'إليكما',
        'إليكن',
        'أم',
        'أما',
        'أما',
        'إما',
        'أن',
        'إن',
        'إنا',
        'أنا',
        'أنت',
        'أنتم',
        'أنتما',
        'أنتن',
        'إنما',
        'إنه',
        'أنى',
        'أنى',
        'آه',
        'آها',
        'أو',
        'أولاء',
        'أولئك',
        'أوه',
        'آي',
        'أي',
        'أيها',
        'إي',
        'أين',
        'أين',
        'أينما',
        'إيه',
        'بخ',
        'بس',
        'بعد',
        'بعض',
        'بك',
        'بكم',
        'بكم',
        'بكما',
        'بكن',
        'بل',
        'بلى',
        'بما',
        'بماذا',
        'بمن',
        'بنا',
        'به',
        'بها',
        'بهم',
        'بهما',
        'بهن',
        'بي',
        'بين',
        'بيد',
        'تلك',
        'تلكم',
        'تلكما',
        'ته',
        'تي',
        'تين',
        'تينك',
        'ثم',
        'ثمة',
        'حاشا',
        'حبذا',
        'حتى',
        'حيث',
        'حيثما',
        'حين',
        'خلا',
        'دون',
        'ذا',
        'ذات',
        'ذاك',
        'ذان',
        'ذانك',
        'ذلك',
        'ذلكم',
        'ذلكما',
        'ذلكن',
        'ذه',
        'ذو',
        'ذوا',
        'ذواتا',
        'ذواتي',
        'ذي',
        'ذين',
        'ذينك',
        'ريث',
        'سوف',
        'سوى',
        'شتان',
        'عدا',
        'عسى',
        'عل',
        'على',
        'عليك',
        'عليه',
        'عما',
        'عن',
        'عند',
        'غير',
        'فإذا',
        'فإن',
        'فلا',
        'فمن',
        'في',
        'فيم',
        'فيما',
        'فيه',
        'فيها',
        'قد',
        'كأن',
        'كأنما',
        'كأي',
        'كأين',
        'كذا',
        'كذلك',
        'كل',
        'كلا',
        'كلاهما',
        'كلتا',
        'كلما',
        'كليكما',
        'كليهما',
        'كم',
        'كم',
        'كما',
        'كي',
        'كيت',
        'كيف',
        'كيفما',
        'لا',
        'لاسيما',
        'لدى',
        'لست',
        'لستم',
        'لستما',
        'لستن',
        'لسن',
        'لسنا',
        'لعل',
        'لك',
        'لكم',
        'لكما',
        'لكن',
        'لكنما',
        'لكي',
        'لكيلا',
        'لم',
        'لما',
        'لن',
        'لنا',
        'له',
        'لها',
        'لهم',
        'لهما',
        'لهن',
        'لو',
        'لولا',
        'لوما',
        'لي',
        'لئن',
        'ليت',
        'ليس',
        'ليسا',
        'ليست',
        'ليستا',
        'ليسوا',
        'ما',
        'ماذا',
        'متى',
        'مذ',
        'مع',
        'مما',
        'ممن',
        'من',
        'منه',
        'منها',
        'منذ',
        'مه',
        'مهما',
        'نحن',
        'نحو',
        'نعم',
        'ها',
        'هاتان',
        'هاته',
        'هاتي',
        'هاتين',
        'هاك',
        'هاهنا',
        'هذا',
        'هذان',
        'هذه',
        'هذي',
        'هذين',
        'هكذا',
        'هل',
        'هلا',
        'هم',
        'هما',
        'هن',
        'هنا',
        'هناك',
        'هنالك',
        'هو',
        'هؤلاء',
        'هي',
        'هيا',
        'هيت',
        'هيهات',
        'والذي',
        'والذين',
        'وإذ',
        'وإذا',
        'وإن',
        'ولا',
        'ولكن',
        'ولو',
        'وما',
        'ومن',
        'وهو',
        'يا'
    ],
    'azerbaijani':
    [
        'a',
        'ad',
        'altı',
        'altmış',
        'amma',
        'arasında',
        'artıq',
        'ay',
        'az',
        'bax',
        'belə',
        'bəli',
        'bəlkə',
        'beş',
        'bəy',
        'bəzən',
        'bəzi',
        'bilər',
        'bir',
        'biraz',
        'biri',
        'birşey',
        'biz',
        'bizim',
        'bizlər',
        'bu',
        'buna',
        'bundan',
        'bunların',
        'bunu',
        'bunun',
        'buradan',
        'bütün',
        'ci',
        'cı',
        'çox',
        'cu',
        'cü',
        'çünki',
        'da',
        'daha',
        'də',
        'dedi',
        'dək',
        'dən',
        'dəqiqə',
        'deyil',
        'dir',
        'doqquz',
        'doqsan',
        'dörd',
        'düz',
        'ə',
        'edən',
        'edir',
        'əgər',
        'əlbəttə',
        'elə',
        'əlli',
        'ən',
        'əslində',
        'et',
        'etdi',
        'etmə',
        'etmək',
        'faiz',
        'gilə',
        'görə',
        'ha',
        'haqqında',
        'harada',
        'hə',
        'heç',
        'həm',
        'həmin',
        'həmişə',
        'hər',
        'ı',
        'idi',
        'iki',
        'il',
        'ildə',
        'ilə',
        'ilk',
        'in',
        'indi',
        'isə',
        'istifadə',
        'iyirmi',
        'ki',
        'kim',
        'kimə',
        'kimi',
        'lakin',
        'lap',
        'məhz',
        'mən',
        'mənə',
        'mirşey',
        'nə',
        'nəhayət',
        'niyə',
        'o',
        'obirisi',
        'of',
        'olan',
        'olar',
        'olaraq',
        'oldu',
        'olduğu',
        'olmadı',
        'olmaz',
        'olmuşdur',
        'olsun',
        'olur',
        'on',
        'ona',
        'ondan',
        'onlar',
        'onlardan',
        'onların',
        'onsuzda',
        'onu',
        'onun',
        'oradan',
        'otuz',
        'öz',
        'özü',
        'qarşı',
        'qədər',
        'qırx',
        'saat',
        'sadəcə',
        'saniyə',
        'səhv',
        'səkkiz',
        'səksən',
        'sən',
        'sənə',
        'sənin',
        'siz',
        'sizin',
        'sizlər',
        'sonra',
        'təəssüf',
        'ü',
        'üç',
        'üçün',
        'var',
        'və',
        'xan',
        'xanım',
        'xeyr',
        'ya',
        'yalnız',
        'yaxşı',
        'yeddi',
        'yenə',
        'yəni',
        'yetmiş',
        'yox',
        'yoxdur',
        'yoxsa',
        'yüz',
        'zaman'
    ],
    'danish':
    [
        'og',
        'i',
        'jeg',
        'det',
        'at',
        'en',
        'den',
        'til',
        'er',
        'som',
        'på',
        'de',
        'med',
        'han',
        'af',
        'for',
        'ikke',
        'der',
        'var',
        'mig',
        'sig',
        'men',
        'et',
        'har',
        'om',
        'vi',
        'min',
        'havde',
        'ham',
        'hun',
        'nu',
        'over',
        'da',
        'fra',
        'du',
        'ud',
        'sin',
        'dem',
        'os',
        'op',
        'man',
        'hans',
        'hvor',
        'eller',
        'hvad',
        'skal',
        'selv',
        'her',
        'alle',
        'vil',
        'blev',
        'kunne',
        'ind',
        'når',
        'være',
        'dog',
        'noget',
        'ville',
        'jo',
        'deres',
        'efter',
        'ned',
        'skulle',
        'denne',
        'end',
        'dette',
        'mit',
        'også',
        'under',
        'have',
        'dig',
        'anden',
        'hende',
        'mine',
        'alt',
        'meget',
        'sit',
        'sine',
        'vor',
        'mod',
        'disse',
        'hvis',
        'din',
        'nogle',
        'hos',
        'blive',
        'mange',
        'ad',
        'bliver',
        'hendes',
        'været',
        'thi',
        'jer',
        'sådan'
    ],
    'dutch':
    [
        'de',
        'en',
        'van',
        'ik',
        'te',
        'dat',
        'die',
        'in',
        'een',
        'hij',
        'het',
        'niet',
        'zijn',
        'is',
        'was',
        'op',
        'aan',
        'met',
        'als',
        'voor',
        'had',
        'er',
        'maar',
        'om',
        'hem',
        'dan',
        'zou',
        'of',
        'wat',
        'mijn',
        'men',
        'dit',
        'zo',
        'door',
        'over',
        'ze',
        'zich',
        'bij',
        'ook',
        'tot',
        'je',
        'mij',
        'uit',
        'der',
        'daar',
        'haar',
        'naar',
        'heb',
        'hoe',
        'heeft',
        'hebben',
        'deze',
        'u',
        'want',
        'nog',
        'zal',
        'me',
        'zij',
        'nu',
        'ge',
        'geen',
        'omdat',
        'iets',
        'worden',
        'toch',
        'al',
        'waren',
        'veel',
        'meer',
        'doen',
        'toen',
        'moet',
        'ben',
        'zonder',
        'kan',
        'hun',
        'dus',
        'alles',
        'onder',
        'ja',
        'eens',
        'hier',
        'wie',
        'werd',
        'altijd',
        'doch',
        'wordt',
        'wezen',
        'kunnen',
        'ons',
        'zelf',
        'tegen',
        'na',
        'reeds',
        'wil',
        'kon',
        'niets',
        'uw',
        'iemand',
        'geweest',
        'andere'
    ],
    'english':
    [
        'i',
        'me',
        'my',
        'myself',
        'we',
        'our',
        'ours',
        'ourselves',
        'you',
        "you're",
        "you've",
        "you'll",
        "you'd",
        'your',
        'yours',
        'yourself',
        'yourselves',
        'he',
        'him',
        'his',
        'himself',
        'she',
        "she's",
        'her',
        'hers',
        'herself',
        'it',
        "it's",
        'its',
        'itself',
        'they',
        'them',
        'their',
        'theirs',
        'themselves',
        'what',
        'which',
        'who',
        'whom',
        'this',
        'that',
        "that'll",
        'these',
        'those',
        'am',
        'is',
        'are',
        'was',
        'were',
        'be',
        'been',
        'being',
        'have',
        'has',
        'had',
        'having',
        'do',
        'does',
        'did',
        'doing',
        'a',
        'an',
        'the',
        'and',
        'but',
        'if',
        'or',
        'because',
        'as',
        'until',
        'while',
        'of',
        'at',
        'by',
        'for',
        'with',
        'about',
        'against',
        'between',
        'into',
        'through',
        'during',
        'before',
        'after',
        'above',
        'below',
        'to',
        'from',
        'up',
        'down',
        'in',
        'out',
        'on',
        'off',
        'over',
        'under',
        'again',
        'further',
        'then',
        'once',
        'here',
        'there',
        'when',
        'where',
        'why',
        'how',
        'all',
        'any',
        'both',
        'each',
        'few',
        'more',
        'most',
        'other',
        'some',
        'such',
        'no',
        'nor',
        'not',
        'only',
        'own',
        'same',
        'so',
        'than',
        'too',
        'very',
        's',
        't',
        'can',
        'will',
        'just',
        'don',
        "don't",
        'should',
        "should've",
        'now',
        'd',
        'll',
        'm',
        'o',
        're',
        've',
        'y',
        'ain',
        'aren',
        "aren't",
        'couldn',
        "couldn't",
        'didn',
        "didn't",
        'doesn',
        "doesn't",
        'hadn',
        "hadn't",
        'hasn',
        "hasn't",
        'haven',
        "haven't",
        'isn',
        "isn't",
        'ma',
        'mightn',
        "mightn't",
        'mustn',
        "mustn't",
        'needn',
        "needn't",
        'shan',
        "shan't",
        'shouldn',
        "shouldn't",
        'wasn',
        "wasn't",
        'weren',
        "weren't",
        'won',
        "won't",
        'wouldn',
        "wouldn't"
    ],
    'finnish':
    [
        'olla',
        'olen',
        'olet',
        'on',
        'olemme',
        'olette',
        'ovat',
        'ole',
        'oli',
        'olisi',
        'olisit',
        'olisin',
        'olisimme',
        'olisitte',
        'olisivat',
        'olit',
        'olin',
        'olimme',
        'olitte',
        'olivat',
        'ollut',
        'olleet',
        'en',
        'et',
        'ei',
        'emme',
        'ette',
        'eivät',
        'minä',
        'minun',
        'minut',
        'minua',
        'minussa',
        'minusta',
        'minuun',
        'minulla',
        'minulta',
        'minulle',
        'sinä',
        'sinun',
        'sinut',
        'sinua',
        'sinussa',
        'sinusta',
        'sinuun',
        'sinulla',
        'sinulta',
        'sinulle',
        'hän',
        'hänen',
        'hänet',
        'häntä',
        'hänessä',
        'hänestä',
        'häneen',
        'hänellä',
        'häneltä',
        'hänelle',
        'me',
        'meidän',
        'meidät',
        'meitä',
        'meissä',
        'meistä',
        'meihin',
        'meillä',
        'meiltä',
        'meille',
        'te',
        'teidän',
        'teidät',
        'teitä',
        'teissä',
        'teistä',
        'teihin',
        'teillä',
        'teiltä',
        'teille',
        'he',
        'heidän',
        'heidät',
        'heitä',
        'heissä',
        'heistä',
        'heihin',
        'heillä',
        'heiltä',
        'heille',
        'tämä',
        'tämän',
        'tätä',
        'tässä',
        'tästä',
        'tähän',
        'tallä',
        'tältä',
        'tälle',
        'tänä',
        'täksi',
        'tuo',
        'tuon',
        'tuotä',
        'tuossa',
        'tuosta',
        'tuohon',
        'tuolla',
        'tuolta',
        'tuolle',
        'tuona',
        'tuoksi',
        'se',
        'sen',
        'sitä',
        'siinä',
        'siitä',
        'siihen',
        'sillä',
        'siltä',
        'sille',
        'sinä',
        'siksi',
        'nämä',
        'näiden',
        'näitä',
        'näissä',
        'näistä',
        'näihin',
        'näillä',
        'näiltä',
        'näille',
        'näinä',
        'näiksi',
        'nuo',
        'noiden',
        'noita',
        'noissa',
        'noista',
        'noihin',
        'noilla',
        'noilta',
        'noille',
        'noina',
        'noiksi',
        'ne',
        'niiden',
        'niitä',
        'niissä',
        'niistä',
        'niihin',
        'niillä',
        'niiltä',
        'niille',
        'niinä',
        'niiksi',
        'kuka',
        'kenen',
        'kenet',
        'ketä',
        'kenessä',
        'kenestä',
        'keneen',
        'kenellä',
        'keneltä',
        'kenelle',
        'kenenä',
        'keneksi',
        'ketkä',
        'keiden',
        'ketkä',
        'keitä',
        'keissä',
        'keistä',
        'keihin',
        'keillä',
        'keiltä',
        'keille',
        'keinä',
        'keiksi',
        'mikä',
        'minkä',
        'minkä',
        'mitä',
        'missä',
        'mistä',
        'mihin',
        'millä',
        'miltä',
        'mille',
        'minä',
        'miksi',
        'mitkä',
        'joka',
        'jonka',
        'jota',
        'jossa',
        'josta',
        'johon',
        'jolla',
        'jolta',
        'jolle',
        'jona',
        'joksi',
        'jotka',
        'joiden',
        'joita',
        'joissa',
        'joista',
        'joihin',
        'joilla',
        'joilta',
        'joille',
        'joina',
        'joiksi',
        'että',
        'ja',
        'jos',
        'koska',
        'kuin',
        'mutta',
        'niin',
        'sekä',
        'sillä',
        'tai',
        'vaan',
        'vai',
        'vaikka',
        'kanssa',
        'mukaan',
        'noin',
        'poikki',
        'yli',
        'kun',
        'niin',
        'nyt',
        'itse'
    ],
    'french':
    [
        'au',
        'aux',
        'avec',
        'ce',
        'ces',
        'dans',
        'de',
        'des',
        'du',
        'elle',
        'en',
        'et',
        'eux',
        'il',
        'je',
        'la',
        'le',
        'leur',
        'lui',
        'ma',
        'mais',
        'me',
        'même',
        'mes',
        'moi',
        'mon',
        'ne',
        'nos',
        'notre',
        'nous',
        'on',
        'ou',
        'par',
        'pas',
        'pour',
        'qu',
        'que',
        'qui',
        'sa',
        'se',
        'ses',
        'son',
        'sur',
        'ta',
        'te',
        'tes',
        'toi',
        'ton',
        'tu',
        'un',
        'une',
        'vos',
        'votre',
        'vous',
        'c',
        'd',
        'j',
        'l',
        'à',
        'm',
        'n',
        's',
        't',
        'y',
        'été',
        'étée',
        'étées',
        'étés',
        'étant',
        'étante',
        'étants',
        'étantes',
        'suis',
        'es',
        'est',
        'sommes',
        'êtes',
        'sont',
        'serai',
        'seras',
        'sera',
        'serons',
        'serez',
        'seront',
        'serais',
        'serait',
        'serions',
        'seriez',
        'seraient',
        'étais',
        'était',
        'étions',
        'étiez',
        'étaient',
        'fus',
        'fut',
        'fûmes',
        'fûtes',
        'furent',
        'sois',
        'soit',
        'soyons',
        'soyez',
        'soient',
        'fusse',
        'fusses',
        'fût',
        'fussions',
        'fussiez',
        'fussent',
        'ayant',
        'ayante',
        'ayantes',
        'ayants',
        'eu',
        'eue',
        'eues',
        'eus',
        'ai',
        'as',
        'avons',
        'avez',
        'ont',
        'aurai',
        'auras',
        'aura',
        'aurons',
        'aurez',
        'auront',
        'aurais',
        'aurait',
        'aurions',
        'auriez',
        'auraient',
        'avais',
        'avait',
        'avions',
        'aviez',
        'avaient',
        'eut',
        'eûmes',
        'eûtes',
        'eurent',
        'aie',
        'aies',
        'ait',
        'ayons',
        'ayez',
        'aient',
        'eusse',
        'eusses',
        'eût',
        'eussions',
        'eussiez',
        'eussent'
    ],
    'german':
    [
        'aber',
        'alle',
        'allem',
        'allen',
        'aller',
        'alles',
        'als',
        'also',
        'am',
        'an',
        'ander',
        'andere',
        'anderem',
        'anderen',
        'anderer',
        'anderes',
        'anderm',
        'andern',
        'anderr',
        'anders',
        'auch',
        'auf',
        'aus',
        'bei',
        'bin',
        'bis',
        'bist',
        'da',
        'damit',
        'dann',
        'der',
        'den',
        'des',
        'dem',
        'die',
        'das',
        'daß',
        'derselbe',
        'derselben',
        'denselben',
        'desselben',
        'demselben',
        'dieselbe',
        'dieselben',
        'dasselbe',
        'dazu',
        'dein',
        'deine',
        'deinem',
        'deinen',
        'deiner',
        'deines',
        'denn',
        'derer',
        'dessen',
        'dich',
        'dir',
        'du',
        'dies',
        'diese',
        'diesem',
        'diesen',
        'dieser',
        'dieses',
        'doch',
        'dort',
        'durch',
        'ein',
        'eine',
        'einem',
        'einen',
        'einer',
        'eines',
        'einig',
        'einige',
        'einigem',
        'einigen',
        'einiger',
        'einiges',
        'einmal',
        'er',
        'ihn',
        'ihm',
        'es',
        'etwas',
        'euer',
        'eure',
        'eurem',
        'euren',
        'eurer',
        'eures',
        'für',
        'gegen',
        'gewesen',
        'hab',
        'habe',
        'haben',
        'hat',
        'hatte',
        'hatten',
        'hier',
        'hin',
        'hinter',
        'ich',
        'mich',
        'mir',
        'ihr',
        'ihre',
        'ihrem',
        'ihren',
        'ihrer',
        'ihres',
        'euch',
        'im',
        'in',
        'indem',
        'ins',
        'ist',
        'jede',
        'jedem',
        'jeden',
        'jeder',
        'jedes',
        'jene',
        'jenem',
        'jenen',
        'jener',
        'jenes',
        'jetzt',
        'kann',
        'kein',
        'keine',
        'keinem',
        'keinen',
        'keiner',
        'keines',
        'können',
        'könnte',
        'machen',
        'man',
        'manche',
        'manchem',
        'manchen',
        'mancher',
        'manches',
        'mein',
        'meine',
        'meinem',
        'meinen',
        'meiner',
        'meines',
        'mit',
        'muss',
        'musste',
        'nach',
        'nicht',
        'nichts',
        'noch',
        'nun',
        'nur',
        'ob',
        'oder',
        'ohne',
        'sehr',
        'sein',
        'seine',
        'seinem',
        'seinen',
        'seiner',
        'seines',
        'selbst',
        'sich',
        'sie',
        'ihnen',
        'sind',
        'so',
        'solche',
        'solchem',
        'solchen',
        'solcher',
        'solches',
        'soll',
        'sollte',
        'sondern',
        'sonst',
        'über',
        'um',
        'und',
        'uns',
        'unsere',
        'unserem',
        'unseren',
        'unser',
        'unseres',
        'unter',
        'viel',
        'vom',
        'von',
        'vor',
        'während',
        'war',
        'waren',
        'warst',
        'was',
        'weg',
        'weil',
        'weiter',
        'welche',
        'welchem',
        'welchen',
        'welcher',
        'welches',
        'wenn',
        'werde',
        'werden',
        'wie',
        'wieder',
        'will',
        'wir',
        'wird',
        'wirst',
        'wo',
        'wollen',
        'wollte',
        'würde',
        'würden',
        'zu',
        'zum',
        'zur',
        'zwar',
        'zwischen'
    ],
    'greek':
    [
        'αλλα',
        'αν',
        'αντι',
        'απο',
        'αυτα',
        'αυτεσ',
        'αυτη',
        'αυτο',
        'αυτοι',
        'αυτοσ',
        'αυτουσ',
        'αυτων',
        'αἱ',
        'αἳ',
        'αἵ',
        'αὐτόσ',
        'αὐτὸς',
        'αὖ',
        'γάρ',
        'γα',
        'γα^',
        'γε',
        'για',
        'γοῦν',
        'γὰρ',
        "δ'",
        'δέ',
        'δή',
        'δαί',
        'δαίσ',
        'δαὶ',
        'δαὶς',
        'δε',
        'δεν',
        "δι'",
        'διά',
        'διὰ',
        'δὲ',
        'δὴ',
        'δ’',
        'εαν',
        'ειμαι',
        'ειμαστε',
        'ειναι',
        'εισαι',
        'ειστε',
        'εκεινα',
        'εκεινεσ',
        'εκεινη',
        'εκεινο',
        'εκεινοι',
        'εκεινοσ',
        'εκεινουσ',
        'εκεινων',
        'ενω',
        'επ',
        'επι',
        'εἰ',
        'εἰμί',
        'εἰμὶ',
        'εἰς',
        'εἰσ',
        'εἴ',
        'εἴμι',
        'εἴτε',
        'η',
        'θα',
        'ισωσ',
        'κ',
        'καί',
        'καίτοι',
        'καθ',
        'και',
        'κατ',
        'κατά',
        'κατα',
        'κατὰ',
        'καὶ',
        'κι',
        'κἀν',
        'κἂν',
        'μέν',
        'μή',
        'μήτε',
        'μα',
        'με',
        'μεθ',
        'μετ',
        'μετά',
        'μετα',
        'μετὰ',
        'μη',
        'μην',
        'μἐν',
        'μὲν',
        'μὴ',
        'μὴν',
        'να',
        'ο',
        'οι',
        'ομωσ',
        'οπωσ',
        'οσο',
        'οτι',
        'οἱ',
        'οἳ',
        'οἷς',
        'οὐ',
        'οὐδ',
        'οὐδέ',
        'οὐδείσ',
        'οὐδεὶς',
        'οὐδὲ',
        'οὐδὲν',
        'οὐκ',
        'οὐχ',
        'οὐχὶ',
        'οὓς',
        'οὔτε',
        'οὕτω',
        'οὕτως',
        'οὕτωσ',
        'οὖν',
        'οὗ',
        'οὗτος',
        'οὗτοσ',
        'παρ',
        'παρά',
        'παρα',
        'παρὰ',
        'περί',
        'περὶ',
        'ποια',
        'ποιεσ',
        'ποιο',
        'ποιοι',
        'ποιοσ',
        'ποιουσ',
        'ποιων',
        'ποτε',
        'που',
        'ποῦ',
        'προ',
        'προσ',
        'πρόσ',
        'πρὸ',
        'πρὸς',
        'πως',
        'πωσ',
        'σε',
        'στη',
        'στην',
        'στο',
        'στον',
        'σόσ',
        'σύ',
        'σύν',
        'σὸς',
        'σὺ',
        'σὺν',
        'τά',
        'τήν',
        'τί',
        'τίς',
        'τίσ',
        'τα',
        'ταῖς',
        'τε',
        'την',
        'τησ',
        'τι',
        'τινα',
        'τις',
        'τισ',
        'το',
        'τοί',
        'τοι',
        'τοιοῦτος',
        'τοιοῦτοσ',
        'τον',
        'τοτε',
        'του',
        'τούσ',
        'τοὺς',
        'τοῖς',
        'τοῦ',
        'των',
        'τό',
        'τόν',
        'τότε',
        'τὰ',
        'τὰς',
        'τὴν',
        'τὸ',
        'τὸν',
        'τῆς',
        'τῆσ',
        'τῇ',
        'τῶν',
        'τῷ',
        'ωσ',
        "ἀλλ'",
        'ἀλλά',
        'ἀλλὰ',
        'ἀλλ’',
        'ἀπ',
        'ἀπό',
        'ἀπὸ',
        'ἀφ',
        'ἂν',
        'ἃ',
        'ἄλλος',
        'ἄλλοσ',
        'ἄν',
        'ἄρα',
        'ἅμα',
        'ἐάν',
        'ἐγώ',
        'ἐγὼ',
        'ἐκ',
        'ἐμόσ',
        'ἐμὸς',
        'ἐν',
        'ἐξ',
        'ἐπί',
        'ἐπεὶ',
        'ἐπὶ',
        'ἐστι',
        'ἐφ',
        'ἐὰν',
        'ἑαυτοῦ',
        'ἔτι',
        'ἡ',
        'ἢ',
        'ἣ',
        'ἤ',
        'ἥ',
        'ἧς',
        'ἵνα',
        'ὁ',
        'ὃ',
        'ὃν',
        'ὃς',
        'ὅ',
        'ὅδε',
        'ὅθεν',
        'ὅπερ',
        'ὅς',
        'ὅσ',
        'ὅστις',
        'ὅστισ',
        'ὅτε',
        'ὅτι',
        'ὑμόσ',
        'ὑπ',
        'ὑπέρ',
        'ὑπό',
        'ὑπὲρ',
        'ὑπὸ',
        'ὡς',
        'ὡσ',
        'ὥς',
        'ὥστε',
        'ὦ',
        'ᾧ'
    ],
    'hungarian':
    [
        'a',
        'ahogy',
        'ahol',
        'aki',
        'akik',
        'akkor',
        'alatt',
        'által',
        'általában',
        'amely',
        'amelyek',
        'amelyekben',
        'amelyeket',
        'amelyet',
        'amelynek',
        'ami',
        'amit',
        'amolyan',
        'amíg',
        'amikor',
        'át',
        'abban',
        'ahhoz',
        'annak',
        'arra',
        'arról',
        'az',
        'azok',
        'azon',
        'azt',
        'azzal',
        'azért',
        'aztán',
        'azután',
        'azonban',
        'bár',
        'be',
        'belül',
        'benne',
        'cikk',
        'cikkek',
        'cikkeket',
        'csak',
        'de',
        'e',
        'eddig',
        'egész',
        'egy',
        'egyes',
        'egyetlen',
        'egyéb',
        'egyik',
        'egyre',
        'ekkor',
        'el',
        'elég',
        'ellen',
        'elõ',
        'elõször',
        'elõtt',
        'elsõ',
        'én',
        'éppen',
        'ebben',
        'ehhez',
        'emilyen',
        'ennek',
        'erre',
        'ez',
        'ezt',
        'ezek',
        'ezen',
        'ezzel',
        'ezért',
        'és',
        'fel',
        'felé',
        'hanem',
        'hiszen',
        'hogy',
        'hogyan',
        'igen',
        'így',
        'illetve',
        'ill.',
        'ill',
        'ilyen',
        'ilyenkor',
        'ison',
        'ismét',
        'itt',
        'jó',
        'jól',
        'jobban',
        'kell',
        'kellett',
        'keresztül',
        'keressünk',
        'ki',
        'kívül',
        'között',
        'közül',
        'legalább',
        'lehet',
        'lehetett',
        'legyen',
        'lenne',
        'lenni',
        'lesz',
        'lett',
        'maga',
        'magát',
        'majd',
        'majd',
        'már',
        'más',
        'másik',
        'meg',
        'még',
        'mellett',
        'mert',
        'mely',
        'melyek',
        'mi',
        'mit',
        'míg',
        'miért',
        'milyen',
        'mikor',
        'minden',
        'mindent',
        'mindenki',
        'mindig',
        'mint',
        'mintha',
        'mivel',
        'most',
        'nagy',
        'nagyobb',
        'nagyon',
        'ne',
        'néha',
        'nekem',
        'neki',
        'nem',
        'néhány',
        'nélkül',
        'nincs',
        'olyan',
        'ott',
        'össze',
        'õ',
        'õk',
        'õket',
        'pedig',
        'persze',
        'rá',
        's',
        'saját',
        'sem',
        'semmi',
        'sok',
        'sokat',
        'sokkal',
        'számára',
        'szemben',
        'szerint',
        'szinte',
        'talán',
        'tehát',
        'teljes',
        'tovább',
        'továbbá',
        'több',
        'úgy',
        'ugyanis',
        'új',
        'újabb',
        'újra',
        'után',
        'utána',
        'utolsó',
        'vagy',
        'vagyis',
        'valaki',
        'valami',
        'valamint',
        'való',
        'vagyok',
        'van',
        'vannak',
        'volt',
        'voltam',
        'voltak',
        'voltunk',
        'vissza',
        'vele',
        'viszont',
        'volna'
    ],
    'italian':
    [
        'ad',
        'al',
        'allo',
        'ai',
        'agli',
        'all',
        'agl',
        'alla',
        'alle',
        'con',
        'col',
        'coi',
        'da',
        'dal',
        'dallo',
        'dai',
        'dagli',
        'dall',
        'dagl',
        'dalla',
        'dalle',
        'di',
        'del',
        'dello',
        'dei',
        'degli',
        'dell',
        'degl',
        'della',
        'delle',
        'in',
        'nel',
        'nello',
        'nei',
        'negli',
        'nell',
        'negl',
        'nella',
        'nelle',
        'su',
        'sul',
        'sullo',
        'sui',
        'sugli',
        'sull',
        'sugl',
        'sulla',
        'sulle',
        'per',
        'tra',
        'contro',
        'io',
        'tu',
        'lui',
        'lei',
        'noi',
        'voi',
        'loro',
        'mio',
        'mia',
        'miei',
        'mie',
        'tuo',
        'tua',
        'tuoi',
        'tue',
        'suo',
        'sua',
        'suoi',
        'sue',
        'nostro',
        'nostra',
        'nostri',
        'nostre',
        'vostro',
        'vostra',
        'vostri',
        'vostre',
        'mi',
        'ti',
        'ci',
        'vi',
        'lo',
        'la',
        'li',
        'le',
        'gli',
        'ne',
        'il',
        'un',
        'uno',
        'una',
        'ma',
        'ed',
        'se',
        'perché',
        'anche',
        'come',
        'dov',
        'dove',
        'che',
        'chi',
        'cui',
        'non',
        'più',
        'quale',
        'quanto',
        'quanti',
        'quanta',
        'quante',
        'quello',
        'quelli',
        'quella',
        'quelle',
        'questo',
        'questi',
        'questa',
        'queste',
        'si',
        'tutto',
        'tutti',
        'a',
        'c',
        'e',
        'i',
        'l',
        'o',
        'ho',
        'hai',
        'ha',
        'abbiamo',
        'avete',
        'hanno',
        'abbia',
        'abbiate',
        'abbiano',
        'avrò',
        'avrai',
        'avrà',
        'avremo',
        'avrete',
        'avranno',
        'avrei',
        'avresti',
        'avrebbe',
        'avremmo',
        'avreste',
        'avrebbero',
        'avevo',
        'avevi',
        'aveva',
        'avevamo',
        'avevate',
        'avevano',
        'ebbi',
        'avesti',
        'ebbe',
        'avemmo',
        'aveste',
        'ebbero',
        'avessi',
        'avesse',
        'avessimo',
        'avessero',
        'avendo',
        'avuto',
        'avuta',
        'avuti',
        'avute',
        'sono',
        'sei',
        'è',
        'siamo',
        'siete',
        'sia',
        'siate',
        'siano',
        'sarò',
        'sarai',
        'sarà',
        'saremo',
        'sarete',
        'saranno',
        'sarei',
        'saresti',
        'sarebbe',
        'saremmo',
        'sareste',
        'sarebbero',
        'ero',
        'eri',
        'era',
        'eravamo',
        'eravate',
        'erano',
        'fui',
        'fosti',
        'fu',
        'fummo',
        'foste',
        'furono',
        'fossi',
        'fosse',
        'fossimo',
        'fossero',
        'essendo',
        'faccio',
        'fai',
        'facciamo',
        'fanno',
        'faccia',
        'facciate',
        'facciano',
        'farò',
        'farai',
        'farà',
        'faremo',
        'farete',
        'faranno',
        'farei',
        'faresti',
        'farebbe',
        'faremmo',
        'fareste',
        'farebbero',
        'facevo',
        'facevi',
        'faceva',
        'facevamo',
        'facevate',
        'facevano',
        'feci',
        'facesti',
        'fece',
        'facemmo',
        'faceste',
        'fecero',
        'facessi',
        'facesse',
        'facessimo',
        'facessero',
        'facendo',
        'sto',
        'stai',
        'sta',
        'stiamo',
        'stanno',
        'stia',
        'stiate',
        'stiano',
        'starò',
        'starai',
        'starà',
        'staremo',
        'starete',
        'staranno',
        'starei',
        'staresti',
        'starebbe',
        'staremmo',
        'stareste',
        'starebbero',
        'stavo',
        'stavi',
        'stava',
        'stavamo',
        'stavate',
        'stavano',
        'stetti',
        'stesti',
        'stette',
        'stemmo',
        'steste',
        'stettero',
        'stessi',
        'stesse',
        'stessimo',
        'stessero',
        'stando'
    ],
    'kazakh':
    [
        'ах',
        'ох',
        'эх',
        'ай',
        'эй',
        'ой',
        'тағы',
        'тағыда',
        'әрине',
        'жоқ',
        'сондай',
        'осындай',
        'осылай',
        'солай',
        'мұндай',
        'бұндай',
        'мен',
        'сен',
        'ол',
        'біз',
        'біздер',
        'олар',
        'сіз',
        'сіздер',
        'маған',
        'оған',
        'саған',
        'біздің',
        'сіздің',
        'оның',
        'бізге',
        'сізге',
        'оларға',
        'біздерге',
        'сіздерге',
        'оларға',
        'менімен',
        'сенімен',
        'онымен',
        'бізбен',
        'сізбен',
        'олармен',
        'біздермен',
        'сіздермен',
        'менің',
        'сенің',
        'біздің',
        'сіздің',
        'оның',
        'біздердің',
        'сіздердің',
        'олардың',
        'маған',
        'саған',
        'оған',
        'менен',
        'сенен',
        'одан',
        'бізден',
        'сізден',
        'олардан',
        'біздерден',
        'сіздерден',
        'олардан',
        'айтпақшы',
        'сонымен',
        'сондықтан',
        'бұл',
        'осы',
        'сол',
        'анау',
        'мынау',
        'сонау',
        'осынау',
        'ана',
        'мына',
        'сона',
        'әні',
        'міне',
        'өй',
        'үйт',
        'бүйт',
        'біреу',
        'кейбіреу',
        'кейбір',
        'қайсыбір',
        'әрбір',
        'бірнеше',
        'бірдеме',
        'бірнеше',
        'әркім',
        'әрне',
        'әрқайсы',
        'әрқалай',
        'әлдекім',
        'әлдене',
        'әлдеқайдан',
        'әлденеше',
        'әлдеқалай',
        'әлдеқашан',
        'алдақашан',
        'еш',
        'ешкім',
        'ешбір',
        'ештеме',
        'дәнеңе',
        'ешқашан',
        'ешқандай',
        'ешқайсы',
        'емес',
        'бәрі',
        'барлық',
        'барша',
        'бар',
        'күллі',
        'бүкіл',
        'түгел',
        'өз',
        'өзім',
        'өзің',
        'өзінің',
        'өзіме',
        'өзіне',
        'өзімнің',
        'өзі',
        'өзге',
        'менде',
        'сенде',
        'онда',
        'менен',
        'сенен\tонан',
        'одан',
        'ау',
        'па',
        'ей',
        'әй',
        'е',
        'уа',
        'уау',
        'уай',
        'я',
        'пай',
        'ә',
        'о',
        'оһо',
        'ой',
        'ие',
        'аһа',
        'ау',
        'беу',
        'мәссаған',
        'бәрекелді',
        'әттегенай',
        'жаракімалла',
        'масқарай',
        'астапыралла',
        'япырмай',
        'ойпырмай',
        'кәне',
        'кәнеки',
        'ал',
        'әйда',
        'кәні',
        'міне',
        'әні',
        'сорап',
        'қош-қош',
        'пфша',
        'пішә',
        'құрау-құрау',
        'шәйт',
        'шек',
        'моһ',
        'тәк',
        'құрау',
        'құр',
        'кә',
        'кәһ',
        'күшім',
        'күшім',
        'мышы',
        'пырс',
        'әукім',
        'алақай',
        'паһ-паһ',
        'бәрекелді',
        'ура',
        'әттең',
        'әттеген-ай',
        'қап',
        'түге',
        'пішту',
        'шіркін',
        'алатау',
        'пай-пай',
        'үшін',
        'сайын',
        'сияқты',
        'туралы',
        'арқылы',
        'бойы',
        'бойымен',
        'шамалы',
        'шақты',
        'қаралы',
        'ғұрлы',
        'ғұрлым',
        'шейін',
        'дейін',
        'қарай',
        'таман',
        'салым',
        'тарта',
        'жуық',
        'таяу',
        'гөрі',
        'бері',
        'кейін',
        'соң',
        'бұрын',
        'бетер',
        'қатар',
        'бірге',
        'қоса',
        'арс',
        'гүрс',
        'дүрс',
        'қорс',
        'тарс',
        'тырс',
        'ырс',
        'барқ',
        'борт',
        'күрт',
        'кірт',
        'морт',
        'сарт',
        'шырт',
        'дүңк',
        'күңк',
        'қыңқ',
        'мыңқ',
        'маңқ',
        'саңқ',
        'шаңқ',
        'шіңк',
        'сыңқ',
        'таңқ',
        'тыңқ',
        'ыңқ',
        'болп',
        'былп',
        'жалп',
        'желп',
        'қолп',
        'ірк',
        'ырқ',
        'сарт-сұрт',
        'тарс-тұрс',
        'арс-ұрс',
        'жалт-жалт',
        'жалт-жұлт',
        'қалт-қалт',
        'қалт-құлт',
        'қаңқ-қаңқ',
        'қаңқ-құңқ',
        'шаңқ-шаңқ',
        'шаңқ-шұңқ',
        'арбаң-арбаң',
        'бүгжең-бүгжең',
        'арсалаң-арсалаң',
        'ербелең-ербелең',
        'батыр-бұтыр',
        'далаң-далаң',
        'тарбаң-тарбаң',
        'қызараң-қызараң',
        'қаңғыр-күңгір',
        'қайқаң-құйқаң',
        'митың-митың',
        'салаң-сұлаң',
        'ыржың-тыржың',
        'бірақ',
        'алайда',
        'дегенмен',
        'әйтпесе',
        'әйткенмен',
        'себебі',
        'өйткені',
        'сондықтан',
        'үшін',
        'сайын',
        'сияқты',
        'туралы',
        'арқылы',
        'бойы',
        'бойымен',
        'шамалы',
        'шақты',
        'қаралы',
        'ғұрлы',
        'ғұрлым',
        'гөрі',
        'бері',
        'кейін',
        'соң',
        'бұрын',
        'бетер',
        'қатар',
        'бірге',
        'қоса',
        'шейін',
        'дейін',
        'қарай',
        'таман',
        'салым',
        'тарта',
        'жуық',
        'таяу',
        'арнайы',
        'осындай',
        'ғана',
        'қана',
        'тек',
        'әншейін'
    ],
    'nepali':
    [
        'छ',
        'र',
        'पनि',
        'छन्',
        'लागि',
        'भएको',
        'गरेको',
        'भने',
        'गर्न',
        'गर्ने',
        'हो',
        'तथा',
        'यो',
        'रहेको',
        'उनले',
        'थियो',
        'हुने',
        'गरेका',
        'थिए',
        'गर्दै',
        'तर',
        'नै',
        'को',
        'मा',
        'हुन्',
        'भन्ने',
        'हुन',
        'गरी',
        'त',
        'हुन्छ',
        'अब',
        'के',
        'रहेका',
        'गरेर',
        'छैन',
        'दिए',
        'भए',
        'यस',
        'ले',
        'गर्नु',
        'औं',
        'सो',
        'त्यो',
        'कि',
        'जुन',
        'यी',
        'का',
        'गरि',
        'ती',
        'न',
        'छु',
        'छौं',
        'लाई',
        'नि',
        'उप',
        'अक्सर',
        'आदि',
        'कसरी',
        'क्रमशः',
        'चाले',
        'अगाडी',
        'अझै',
        'अनुसार',
        'अन्तर्गत',
        'अन्य',
        'अन्यत्र',
        'अन्यथा',
        'अरु',
        'अरुलाई',
        'अर्को',
        'अर्थात',
        'अर्थात्',
        'अलग',
        'आए',
        'आजको',
        'ओठ',
        'आत्म',
        'आफू',
        'आफूलाई',
        'आफ्नै',
        'आफ्नो',
        'आयो',
        'उदाहरण',
        'उनको',
        'उहालाई',
        'एउटै',
        'एक',
        'एकदम',
        'कतै',
        'कम से कम',
        'कसै',
        'कसैले',
        'कहाँबाट',
        'कहिलेकाहीं',
        'का',
        'किन',
        'किनभने',
        'कुनै',
        'कुरा',
        'कृपया',
        'केही',
        'कोही',
        'गए',
        'गरौं',
        'गर्छ',
        'गर्छु',
        'गर्नुपर्छ',
        'गयौ',
        'गैर',
        'चार',
        'चाहनुहुन्छ',
        'चाहन्छु',
        'चाहिए',
        'छू',
        'जताततै',
        'जब',
        'जबकि',
        'जसको',
        'जसबाट',
        'जसमा',
        'जसलाई',
        'जसले',
        'जस्तै',
        'जस्तो',
        'जस्तोसुकै',
        'जहाँ',
        'जान',
        'जाहिर',
        'जे',
        'जो',
        'ठीक',
        'तत्काल',
        'तदनुसार',
        'तपाईको',
        'तपाई',
        'पर्याप्त',
        'पहिले',
        'पहिलो',
        'पहिल्यै',
        'पाँच',
        'पाँचौं',
        'तल',
        'तापनी',
        'तिनी',
        'तिनीहरू',
        'तिनीहरुको',
        'तिनिहरुलाई',
        'तिमी',
        'तिर',
        'तीन',
        'तुरुन्तै',
        'तेस्रो',
        'तेस्कारण',
        'पूर्व',
        'प्रति',
        'प्रतेक',
        'प्लस',
        'फेरी',
        'बने',
        'त्सपछि',
        'त्सैले',
        'त्यहाँ',
        'थिएन',
        'दिनुभएको',
        'दिनुहुन्छ',
        'दुई',
        'देखि',
        'बरु',
        'बारे',
        'बाहिर',
        'देखिन्छ',
        'देखियो',
        'देखे',
        'देखेको',
        'देखेर',
        'दोस्रो',
        'धेरै',
        'नजिकै',
        'नत्र',
        'नयाँ',
        'निम्ति',
        'बाहेक',
        'बीच',
        'बीचमा',
        'भन',
        'निम्न',
        'निम्नानुसार',
        'निर्दिष्ट',
        'नौ',
        'पक्का',
        'पक्कै',
        'पछि',
        'पछिल्लो',
        'पटक',
        'पर्छ',
        'पर्थ्यो',
        'भन्छन्',
        'भन्',
        'भन्छु',
        'भन्दा',
        'भन्नुभयो',
        'भर',
        'भित्र',
        'भित्री',
        'म',
        'मलाई',
        'मात्र',
        'माथि',
        'मुख्य',
        'मेरो',
        'यति',
        'यथोचित',
        'यदि',
        'यद्यपि',
        'यसको',
        'यसपछि',
        'यसबाहेक',
        'यसरी',
        'यसो',
        'यस्तो',
        'यहाँ',
        'यहाँसम्म',
        'या',
        'रही',
        'राखे',
        'राख्छ',
        'राम्रो',
        'रूप',
        'लगभग',
        'वरीपरी',
        'वास्तवमा',
        'बिरुद्ध',
        'बिशेष',
        'सायद',
        'शायद',
        'संग',
        'संगै',
        'सक्छ',
        'सट्टा',
        'सधै',
        'सबै',
        'सबैलाई',
        'समय',
        'सम्भव',
        'सम्म',
        'सही',
        'साँच्चै',
        'सात',
        'साथ',
        'साथै',
        'सारा',
        'सोही',
        'स्पष्ट',
        'हरे',
        'हरेक'
    ],
    'norwegian':
    [
        'og',
        'i',
        'jeg',
        'det',
        'at',
        'en',
        'et',
        'den',
        'til',
        'er',
        'som',
        'på',
        'de',
        'med',
        'han',
        'av',
        'ikke',
        'ikkje',
        'der',
        'så',
        'var',
        'meg',
        'seg',
        'men',
        'ett',
        'har',
        'om',
        'vi',
        'min',
        'mitt',
        'ha',
        'hadde',
        'hun',
        'nå',
        'over',
        'da',
        'ved',
        'fra',
        'du',
        'ut',
        'sin',
        'dem',
        'oss',
        'opp',
        'man',
        'kan',
        'hans',
        'hvor',
        'eller',
        'hva',
        'skal',
        'selv',
        'sjøl',
        'her',
        'alle',
        'vil',
        'bli',
        'ble',
        'blei',
        'blitt',
        'kunne',
        'inn',
        'når',
        'være',
        'kom',
        'noen',
        'noe',
        'ville',
        'dere',
        'som',
        'deres',
        'kun',
        'ja',
        'etter',
        'ned',
        'skulle',
        'denne',
        'for',
        'deg',
        'si',
        'sine',
        'sitt',
        'mot',
        'å',
        'meget',
        'hvorfor',
        'dette',
        'disse',
        'uten',
        'hvordan',
        'ingen',
        'din',
        'ditt',
        'blir',
        'samme',
        'hvilken',
        'hvilke',
        'sånn',
        'inni',
        'mellom',
        'vår',
        'hver',
        'hvem',
        'vors',
        'hvis',
        'både',
        'bare',
        'enn',
        'fordi',
        'før',
        'mange',
        'også',
        'slik',
        'vært',
        'være',
        'båe',
        'begge',
        'siden',
        'dykk',
        'dykkar',
        'dei',
        'deira',
        'deires',
        'deim',
        'di',
        'då',
        'eg',
        'ein',
        'eit',
        'eitt',
        'elles',
        'honom',
        'hjå',
        'ho',
        'hoe',
        'henne',
        'hennar',
        'hennes',
        'hoss',
        'hossen',
        'ikkje',
        'ingi',
        'inkje',
        'korleis',
        'korso',
        'kva',
        'kvar',
        'kvarhelst',
        'kven',
        'kvi',
        'kvifor',
        'me',
        'medan',
        'mi',
        'mine',
        'mykje',
        'no',
        'nokon',
        'noka',
        'nokor',
        'noko',
        'nokre',
        'si',
        'sia',
        'sidan',
        'so',
        'somt',
        'somme',
        'um',
        'upp',
        'vere',
        'vore',
        'verte',
        'vort',
        'varte',
        'vart'
    ],
    'portuguese':
    [
        'de',
        'a',
        'o',
        'que',
        'e',
        'do',
        'da',
        'em',
        'um',
        'para',
        'com',
        'não',
        'uma',
        'os',
        'no',
        'se',
        'na',
        'por',
        'mais',
        'as',
        'dos',
        'como',
        'mas',
        'ao',
        'ele',
        'das',
        'à',
        'seu',
        'sua',
        'ou',
        'quando',
        'muito',
        'nos',
        'já',
        'eu',
        'também',
        'só',
        'pelo',
        'pela',
        'até',
        'isso',
        'ela',
        'entre',
        'depois',
        'sem',
        'mesmo',
        'aos',
        'seus',
        'quem',
        'nas',
        'me',
        'esse',
        'eles',
        'você',
        'essa',
        'num',
        'nem',
        'suas',
        'meu',
        'às',
        'minha',
        'numa',
        'pelos',
        'elas',
        'qual',
        'nós',
        'lhe',
        'deles',
        'essas',
        'esses',
        'pelas',
        'este',
        'dele',
        'tu',
        'te',
        'vocês',
        'vos',
        'lhes',
        'meus',
        'minhas',
        'teu',
        'tua',
        'teus',
        'tuas',
        'nosso',
        'nossa',
        'nossos',
        'nossas',
        'dela',
        'delas',
        'esta',
        'estes',
        'estas',
        'aquele',
        'aquela',
        'aqueles',
        'aquelas',
        'isto',
        'aquilo',
        'estou',
        'está',
        'estamos',
        'estão',
        'estive',
        'esteve',
        'estivemos',
        'estiveram',
        'estava',
        'estávamos',
        'estavam',
        'estivera',
        'estivéramos',
        'esteja',
        'estejamos',
        'estejam',
        'estivesse',
        'estivéssemos',
        'estivessem',
        'estiver',
        'estivermos',
        'estiverem',
        'hei',
        'há',
        'havemos',
        'hão',
        'houve',
        'houvemos',
        'houveram',
        'houvera',
        'houvéramos',
        'haja',
        'hajamos',
        'hajam',
        'houvesse',
        'houvéssemos',
        'houvessem',
        'houver',
        'houvermos',
        'houverem',
        'houverei',
        'houverá',
        'houveremos',
        'houverão',
        'houveria',
        'houveríamos',
        'houveriam',
        'sou',
        'somos',
        'são',
        'era',
        'éramos',
        'eram',
        'fui',
        'foi',
        'fomos',
        'foram',
        'fora',
        'fôramos',
        'seja',
        'sejamos',
        'sejam',
        'fosse',
        'fôssemos',
        'fossem',
        'for',
        'formos',
        'forem',
        'serei',
        'será',
        'seremos',
        'serão',
        'seria',
        'seríamos',
        'seriam',
        'tenho',
        'tem',
        'temos',
        'tém',
        'tinha',
        'tínhamos',
        'tinham',
        'tive',
        'teve',
        'tivemos',
        'tiveram',
        'tivera',
        'tivéramos',
        'tenha',
        'tenhamos',
        'tenham',
        'tivesse',
        'tivéssemos',
        'tivessem',
        'tiver',
        'tivermos',
        'tiverem',
        'terei',
        'terá',
        'teremos',
        'terão',
        'teria',
        'teríamos',
        'teriam'
    ],
    'romanian':
    [
        'a',
        'abia',
        'acea',
        'aceasta',
        'această',
        'aceea',
        'aceeasi',
        'acei',
        'aceia',
        'acel',
        'acela',
        'acelasi',
        'acele',
        'acelea',
        'acest',
        'acesta',
        'aceste',
        'acestea',
        'acestei',
        'acestia',
        'acestui',
        'aceşti',
        'aceştia',
        'adica',
        'ai',
        'aia',
        'aibă',
        'aici',
        'al',
        'ala',
        'ale',
        'alea',
        'alt',
        'alta',
        'altceva',
        'altcineva',
        'alte',
        'altfel',
        'alti',
        'altii',
        'altul',
        'am',
        'anume',
        'apoi',
        'ar',
        'are',
        'as',
        'asa',
        'asta',
        'astea',
        'astfel',
        'asupra',
        'atare',
        'atat',
        'atata',
        'atatea',
        'atatia',
        'ati',
        'atit',
        'atita',
        'atitea',
        'atitia',
        'atunci',
        'au',
        'avea',
        'avem',
        'aveţi',
        'avut',
        'aş',
        'aţi',
        'ba',
        'ca',
        'cam',
        'cand',
        'care',
        'careia',
        'carora',
        'caruia',
        'cat',
        'catre',
        'ce',
        'cea',
        'ceea',
        'cei',
        'ceilalti',
        'cel',
        'cele',
        'celor',
        'ceva',
        'chiar',
        'ci',
        'cind',
        'cine',
        'cineva',
        'cit',
        'cita',
        'cite',
        'citeva',
        'citi',
        'citiva',
        'cu',
        'cui',
        'cum',
        'cumva',
        'cât',
        'câte',
        'câtva',
        'câţi',
        'cînd',
        'cît',
        'cîte',
        'cîtva',
        'cîţi',
        'că',
        'căci',
        'cărei',
        'căror',
        'cărui',
        'către',
        'da',
        'daca',
        'dacă',
        'dar',
        'dat',
        'dată',
        'dau',
        'de',
        'deasupra',
        'deci',
        'decit',
        'deja',
        'desi',
        'despre',
        'deşi',
        'din',
        'dintr',
        'dintr-',
        'dintre',
        'doar',
        'doi',
        'doilea',
        'două',
        'drept',
        'dupa',
        'după',
        'dă',
        'e',
        'ea',
        'ei',
        'el',
        'ele',
        'era',
        'eram',
        'este',
        'eu',
        'eşti',
        'face',
        'fara',
        'fata',
        'fel',
        'fi',
        'fie',
        'fiecare',
        'fii',
        'fim',
        'fiu',
        'fiţi',
        'foarte',
        'fost',
        'fără',
        'i',
        'ia',
        'iar',
        'ii',
        'il',
        'imi',
        'in',
        'inainte',
        'inapoi',
        'inca',
        'incit',
        'insa',
        'intr',
        'intre',
        'isi',
        'iti',
        'la',
        'le',
        'li',
        'lor',
        'lui',
        'lângă',
        'lîngă',
        'm',
        'ma',
        'mai',
        'mea',
        'mei',
        'mele',
        'mereu',
        'meu',
        'mi',
        'mie',
        'mine',
        'mod',
        'mult',
        'multa',
        'multe',
        'multi',
        'multă',
        'mulţi',
        'mâine',
        'mîine',
        'mă',
        'ne',
        'ni',
        'nici',
        'nimeni',
        'nimic',
        'niste',
        'nişte',
        'noastre',
        'noastră',
        'noi',
        'nostri',
        'nostru',
        'nou',
        'noua',
        'nouă',
        'noştri',
        'nu',
        'numai',
        'o',
        'or',
        'ori',
        'oricare',
        'orice',
        'oricine',
        'oricum',
        'oricând',
        'oricât',
        'oricînd',
        'oricît',
        'oriunde',
        'pai',
        'parca',
        'patra',
        'patru',
        'pe',
        'pentru',
        'peste',
        'pic',
        'pina',
        'poate',
        'pot',
        'prea',
        'prima',
        'primul',
        'prin',
        'printr-',
        'putini',
        'puţin',
        'puţina',
        'puţină',
        'până',
        'pînă',
        'sa',
        'sa-mi',
        'sa-ti',
        'sai',
        'sale',
        'sau',
        'se',
        'si',
        'sint',
        'sintem',
        'spate',
        'spre',
        'sub',
        'sunt',
        'suntem',
        'sunteţi',
        'sus',
        'să',
        'săi',
        'său',
        't',
        'ta',
        'tale',
        'te',
        'ti',
        'tine',
        'toata',
        'toate',
        'toată',
        'tocmai',
        'tot',
        'toti',
        'totul',
        'totusi',
        'totuşi',
        'toţi',
        'trei',
        'treia',
        'treilea',
        'tu',
        'tuturor',
        'tăi',
        'tău',
        'u',
        'ul',
        'ului',
        'un',
        'una',
        'unde',
        'undeva',
        'unei',
        'uneia',
        'unele',
        'uneori',
        'unii',
        'unor',
        'unora',
        'unu',
        'unui',
        'unuia',
        'unul',
        'v',
        'va',
        'vi',
        'voastre',
        'voastră',
        'voi',
        'vom',
        'vor',
        'vostru',
        'vouă',
        'voştri',
        'vreo',
        'vreun',
        'vă',
        'zi',
        'zice',
        'îi',
        'îl',
        'îmi',
        'în',
        'îţi',
        'ăla',
        'ălea',
        'ăsta',
        'ăstea',
        'ăştia',
        'şi',
        'ţi',
        'ţie'
    ],
    'russian':
    [
        'и',
        'в',
        'во',
        'не',
        'что',
        'он',
        'на',
        'я',
        'с',
        'со',
        'как',
        'а',
        'то',
        'все',
        'она',
        'так',
        'его',
        'но',
        'да',
        'ты',
        'к',
        'у',
        'же',
        'вы',
        'за',
        'бы',
        'по',
        'только',
        'ее',
        'мне',
        'было',
        'вот',
        'от',
        'меня',
        'еще',
        'нет',
        'о',
        'из',
        'ему',
        'теперь',
        'когда',
        'даже',
        'ну',
        'вдруг',
        'ли',
        'если',
        'уже',
        'или',
        'ни',
        'быть',
        'был',
        'него',
        'до',
        'вас',
        'нибудь',
        'опять',
        'уж',
        'вам',
        'ведь',
        'там',
        'потом',
        'себя',
        'ничего',
        'ей',
        'может',
        'они',
        'тут',
        'где',
        'есть',
        'надо',
        'ней',
        'для',
        'мы',
        'тебя',
        'их',
        'чем',
        'была',
        'сам',
        'чтоб',
        'без',
        'будто',
        'чего',
        'раз',
        'тоже',
        'себе',
        'под',
        'будет',
        'ж',
        'тогда',
        'кто',
        'этот',
        'того',
        'потому',
        'этого',
        'какой',
        'совсем',
        'ним',
        'здесь',
        'этом',
        'один',
        'почти',
        'мой',
        'тем',
        'чтобы',
        'нее',
        'сейчас',
        'были',
        'куда',
        'зачем',
        'всех',
        'никогда',
        'можно',
        'при',
        'наконец',
        'два',
        'об',
        'другой',
        'хоть',
        'после',
        'над',
        'больше',
        'тот',
        'через',
        'эти',
        'нас',
        'про',
        'всего',
        'них',
        'какая',
        'много',
        'разве',
        'три',
        'эту',
        'моя',
        'впрочем',
        'хорошо',
        'свою',
        'этой',
        'перед',
        'иногда',
        'лучше',
        'чуть',
        'том',
        'нельзя',
        'такой',
        'им',
        'более',
        'всегда',
        'конечно',
        'всю',
        'между'
    ],
    'spanish':
    [
        'de',
        'la',
        'que',
        'el',
        'en',
        'y',
        'a',
        'los',
        'del',
        'se',
        'las',
        'por',
        'un',
        'para',
        'con',
        'no',
        'una',
        'su',
        'al',
        'lo',
        'como',
        'más',
        'pero',
        'sus',
        'le',
        'ya',
        'o',
        'este',
        'sí',
        'porque',
        'esta',
        'entre',
        'cuando',
        'muy',
        'sin',
        'sobre',
        'también',
        'me',
        'hasta',
        'hay',
        'donde',
        'quien',
        'desde',
        'todo',
        'nos',
        'durante',
        'todos',
        'uno',
        'les',
        'ni',
        'contra',
        'otros',
        'ese',
        'eso',
        'ante',
        'ellos',
        'e',
        'esto',
        'mí',
        'antes',
        'algunos',
        'qué',
        'unos',
        'yo',
        'otro',
        'otras',
        'otra',
        'él',
        'tanto',
        'esa',
        'estos',
        'mucho',
        'quienes',
        'nada',
        'muchos',
        'cual',
        'poco',
        'ella',
        'estar',
        'estas',
        'algunas',
        'algo',
        'nosotros',
        'mi',
        'mis',
        'tú',
        'te',
        'ti',
        'tu',
        'tus',
        'ellas',
        'nosotras',
        'vosostros',
        'vosostras',
        'os',
        'mío',
        'mía',
        'míos',
        'mías',
        'tuyo',
        'tuya',
        'tuyos',
        'tuyas',
        'suyo',
        'suya',
        'suyos',
        'suyas',
        'nuestro',
        'nuestra',
        'nuestros',
        'nuestras',
        'vuestro',
        'vuestra',
        'vuestros',
        'vuestras',
        'esos',
        'esas',
        'estoy',
        'estás',
        'está',
        'estamos',
        'estáis',
        'están',
        'esté',
        'estés',
        'estemos',
        'estéis',
        'estén',
        'estaré',
        'estarás',
        'estará',
        'estaremos',
        'estaréis',
        'estarán',
        'estaría',
        'estarías',
        'estaríamos',
        'estaríais',
        'estarían',
        'estaba',
        'estabas',
        'estábamos',
        'estabais',
        'estaban',
        'estuve',
        'estuviste',
        'estuvo',
        'estuvimos',
        'estuvisteis',
        'estuvieron',
        'estuviera',
        'estuvieras',
        'estuviéramos',
        'estuvierais',
        'estuvieran',
        'estuviese',
        'estuvieses',
        'estuviésemos',
        'estuvieseis',
        'estuviesen',
        'estando',
        'estado',
        'estada',
        'estados',
        'estadas',
        'estad',
        'he',
        'has',
        'ha',
        'hemos',
        'habéis',
        'han',
        'haya',
        'hayas',
        'hayamos',
        'hayáis',
        'hayan',
        'habré',
        'habrás',
        'habrá',
        'habremos',
        'habréis',
        'habrán',
        'habría',
        'habrías',
        'habríamos',
        'habríais',
        'habrían',
        'había',
        'habías',
        'habíamos',
        'habíais',
        'habían',
        'hube',
        'hubiste',
        'hubo',
        'hubimos',
        'hubisteis',
        'hubieron',
        'hubiera',
        'hubieras',
        'hubiéramos',
        'hubierais',
        'hubieran',
        'hubiese',
        'hubieses',
        'hubiésemos',
        'hubieseis',
        'hubiesen',
        'habiendo',
        'habido',
        'habida',
        'habidos',
        'habidas',
        'soy',
        'eres',
        'es',
        'somos',
        'sois',
        'son',
        'sea',
        'seas',
        'seamos',
        'seáis',
        'sean',
        'seré',
        'serás',
        'será',
        'seremos',
        'seréis',
        'serán',
        'sería',
        'serías',
        'seríamos',
        'seríais',
        'serían',
        'era',
        'eras',
        'éramos',
        'erais',
        'eran',
        'fui',
        'fuiste',
        'fue',
        'fuimos',
        'fuisteis',
        'fueron',
        'fuera',
        'fueras',
        'fuéramos',
        'fuerais',
        'fueran',
        'fuese',
        'fueses',
        'fuésemos',
        'fueseis',
        'fuesen',
        'sintiendo',
        'sentido',
        'sentida',
        'sentidos',
        'sentidas',
        'siente',
        'sentid',
        'tengo',
        'tienes',
        'tiene',
        'tenemos',
        'tenéis',
        'tienen',
        'tenga',
        'tengas',
        'tengamos',
        'tengáis',
        'tengan',
        'tendré',
        'tendrás',
        'tendrá',
        'tendremos',
        'tendréis',
        'tendrán',
        'tendría',
        'tendrías',
        'tendríamos',
        'tendríais',
        'tendrían',
        'tenía',
        'tenías',
        'teníamos',
        'teníais',
        'tenían',
        'tuve',
        'tuviste',
        'tuvo',
        'tuvimos',
        'tuvisteis',
        'tuvieron',
        'tuviera',
        'tuvieras',
        'tuviéramos',
        'tuvierais',
        'tuvieran',
        'tuviese',
        'tuvieses',
        'tuviésemos',
        'tuvieseis',
        'tuviesen',
        'teniendo',
        'tenido',
        'tenida',
        'tenidos',
        'tenidas',
        'tened'
    ],
    'swedish':
    [
        'och',
        'det',
        'att',
        'i',
        'en',
        'jag',
        'hon',
        'som',
        'han',
        'på',
        'den',
        'med',
        'var',
        'sig',
        'för',
        'så',
        'till',
        'är',
        'men',
        'ett',
        'om',
        'hade',
        'de',
        'av',
        'icke',
        'mig',
        'du',
        'henne',
        'då',
        'sin',
        'nu',
        'har',
        'inte',
        'hans',
        'honom',
        'skulle',
        'hennes',
        'där',
        'min',
        'man',
        'ej',
        'vid',
        'kunde',
        'något',
        'från',
        'ut',
        'när',
        'efter',
        'upp',
        'vi',
        'dem',
        'vara',
        'vad',
        'över',
        'än',
        'dig',
        'kan',
        'sina',
        'här',
        'ha',
        'mot',
        'alla',
        'under',
        'någon',
        'eller',
        'allt',
        'mycket',
        'sedan',
        'ju',
        'denna',
        'själv',
        'detta',
        'åt',
        'utan',
        'varit',
        'hur',
        'ingen',
        'mitt',
        'ni',
        'bli',
        'blev',
        'oss',
        'din',
        'dessa',
        'några',
        'deras',
        'blir',
        'mina',
        'samma',
        'vilken',
        'er',
        'sådan',
        'vår',
        'blivit',
        'dess',
        'inom',
        'mellan',
        'sådant',
        'varför',
        'varje',
        'vilka',
        'ditt',
        'vem',
        'vilket',
        'sitta',
        'sådana',
        'vart',
        'dina',
        'vars',
        'vårt',
        'våra',
        'ert',
        'era',
        'vilkas'
    ],
    'turkish':
    [
        'acaba',
        'ama',
        'aslında',
        'az',
        'bazı',
        'belki',
        'biri',
        'birkaç',
        'birşey',
        'biz',
        'bu',
        'çok',
        'çünkü',
        'da',
        'daha',
        'de',
        'defa',
        'diye',
        'eğer',
        'en',
        'gibi',
        'hem',
        'hep',
        'hepsi',
        'her',
        'hiç',
        'için',
        'ile',
        'ise',
        'kez',
        'ki',
        'kim',
        'mı',
        'mu',
        'mü',
        'nasıl',
        'ne',
        'neden',
        'nerde',
        'nerede',
        'nereye',
        'niçin',
        'niye',
        'o',
        'sanki',
        'şey',
        'siz',
        'şu',
        'tüm',
        've',
        'veya',
        'ya',
        'yani'
    ]
}



from advertools.ad_create import ad_create

import pytest


def test_raises_error_for_long_input_strings():
    with pytest.raises(ValueError):
        ad_create('short template {}', ['one', 'two', 'three'],
                  'very long fallback string', 20)
    with pytest.raises(ValueError):
        ad_create('very long template string {}', ['one', 'two', 'three'],
                  'short', 20)


def test_all_replacements_used():
    replacements = ['one', 'two', 'three']
    result = ad_create('Hello {}', replacements, 'fallback', capitalize=False)
    assert all([rep in ' '.join(result) for rep in replacements])


def test_fallback_used_if_string_long():
    replacements = ['one', 'two', 'three hundrend thousand']
    result = ad_create('Hello {}', replacements, 'fallback', max_len=20,
                       capitalize=False)
    assert result == ['Hello one', 'Hello two', 'Hello fallback']


def test_final_string_capitalized_or_not():
    capitalized = ad_create('heLLo {}', ['ONE', 'tWo', 'tHree', 'Four'],
                            'fallback', capitalize=True)
    not_capitalized = ad_create('heLLo {}', ['ONE', 'tWo', 'tHree', 'Four'],
                                'fallback', capitalize=False)
    assert capitalized == ['Hello One', 'Hello Two', 'Hello Three',
                           'Hello Four']
    assert not_capitalized == ['heLLo ONE', 'heLLo tWo', 'heLLo tHree',
                               'heLLo Four']



from collections import defaultdict

import advertools as adv
import pandas as pd


def word_frequency(text_list, num_list, sep=None,
                   rm_words=adv.stopwords['english'], extra_info=False):
    """Count the absolute as well as the weighted frequency of words
    in ``text_list`` (based on ``num_list``).

    :param text_list: iterable of strings.
        Typically short phrases, but could be any list of full blown documents.
        Usually, you would use this to analyze tweets, book titles, URLs, etc.
    :param num_list: iterable of numbers.
        A list of numbers with the same length as ``text_list``, describing a
        certain attribute of these 'documents'; views, retweets, sales, etc.
    :param sep: string.
        The seperator with which to separate words in each document.
        The default uses white spaces, and you can specify another separator.
    :param rm_words: iterable of strings.
        Words to remove from the list 'stop-words'. The default uses
        ``nltk``'s list of English stopwords. To get all available languages
        run ``adv.stopwords.keys()``
    :param extra_info: boolean.
        Whethr or not to give additional columns about the frequencies
    :returns abs_wtd_df: absolute and weighted DataFrame.
        pandas.DataFrame with several metrics calculated. The most important
        are ``abs_freq`` and ``wtd_freq``. These show the difference between the
        number of occurrences of each word together with their respective
        weighted occurrences (frequency vs. weighted frequency). Other metrics
        are also provided. The columns are as follows:

        word: Word.
            Words in the document list each on its own row.
        abs_freq: Absolute frequency.
            The number of occurrences of each word in all the documents.
        wtd_freq: Weighted frequency.
            Every occurrence of ``word`` multiplied by its respective value in
            ``num_list`` provided by the function call.
        rel_value: Relative value.
            ``wtd_freq`` divided by ``abs_freq``, showing the value per
            occurrence of ``word``

        :extra_info:

        abs_perc: Absolute frequency percentage.
            ``abs_freq`` divided by the sum of all occurrences of words.
        abs_perc_cum: Cumulative absolute percentage.
            Cumulative sum of ``abs_perc`` to see how many words form x% of
            the occurrences.
        wtd_freq_perc: ``wtd_freq`` divided by the total weighted frequencies of
            all words.
        wtd_freq_perc_cum: Cumulative weighted frequncy percentage.
            Cumulative sum of ``wtd_freq_perc`` to see how many words form x% of
            the weighted occurrences.

    >>> text_list = ['apple orange', 'apple orange banana',
                     'apple kiwi', 'kiwi mango']
    >>> num_list = [100, 100, 100, 400]

    >>> adv.word_frequency(text_list, num_list)
         word  abs_freq  wtd_freq  rel_value
    0    kiwi         2       500      250.0
    1   mango         1       400      400.0
    2   apple         3       300      100.0
    3  orange         2       200      100.0
    4  banana         1       100      100.0

    "kiwi" occurred twice (abs_freq), one of these phrases has a score of 100,
    and the other 400, so the wtd_freq is the sum (100 + 400 = 500)

    >>> adv.word_frequency(text_list, num_list, extra_info=True)
         word  abs_freq  abs_perc  abs_perc_cum  wtd_freq  wtd_freq_perc  wtd_freq_perc_cum  rel_value
    0    kiwi         2  0.222222      0.222222       500       0.333333           0.333333      250.0
    1   mango         1  0.111111      0.333333       400       0.266667           0.600000      400.0
    2   apple         3  0.333333      0.666667       300       0.200000           0.800000      100.0
    3  orange         2  0.222222      0.888889       200       0.133333           0.933333      100.0
    4  banana         1  0.111111      1.000000       100       0.066667           1.000000      100.0

    This is the same result as above but giving the full DataFrame including
    all columns.
    """
    word_freq = defaultdict(lambda: [0, 0])

    for text, num in zip(text_list, num_list):
        for word in text.split(sep=sep):
            if word.lower() in rm_words:
                continue
            word_freq[word.lower()][0] += 1
            word_freq[word.lower()][1] += num

    columns = ['abs_freq', 'wtd_freq']

    abs_wtd_df = (pd.DataFrame.from_dict(word_freq, orient='index',
                                         columns=columns)
                  .sort_values('wtd_freq', ascending=False)
                  .assign(rel_value=lambda df: df['wtd_freq'] / df['abs_freq'])
                  .round())
    if extra_info:
        abs_wtd_df.insert(1, 'abs_perc', value=abs_wtd_df['abs_freq'] /
                          abs_wtd_df['abs_freq'].sum())
        abs_wtd_df.insert(2, 'abs_perc_cum', abs_wtd_df['abs_perc'].cumsum())
        abs_wtd_df.insert(4, 'wtd_freq_perc', abs_wtd_df['wtd_freq'] /
                          abs_wtd_df['wtd_freq'].sum())
        abs_wtd_df.insert(5, 'wtd_freq_perc_cum',
                          abs_wtd_df['wtd_freq_perc'].cumsum())

    abs_wtd_df = abs_wtd_df.reset_index().rename(columns={'index': 'word'})

    return abs_wtd_df



import string


def ad_create(template, replacements, fallback, max_len=30, capitalize=True):
    """Insert each of the replacement strings in its place within template.

    :param template: a string format template, using braces
    :param replacements: replacements string to be inserted in template
    :param fallback: the string to insert in template in case the replacement
        is too long
    :param max_len: the maximum allowed length of the full string
    :returns formatted: list of strings

    >>> ad_create("Let\'s count {}", ['one', 'two', 'three'], 'one', 20)
    ["Let's count one", "Let's count two", "Let's count three"]

    >>> ad_create(template='My favorite car is {}',
                 replacements=['Toyota', 'BMW', 'Mercedes', 'Lamborghini'],
                 fallback='great',
                 max_len=28)
    ['My favorite car is Toyota', 'My favorite car is BMW',
    'My favorite car is Mercedes', 'My favorite car is great']
    """
    if len(template.format(fallback)) > max_len:
        raise ValueError('template + fallback should be <= '
                         + str(max_len) + ' chars')
    final_ad = []
    for rep in replacements:
        if len(template.format(rep)) <= max_len:
            final_ad.append(template.format(rep))
        else:
            final_ad.append(template.format(fallback))

    return [string.capwords(s) for s in final_ad] if capitalize else final_ad




"""Top-level package for advertools."""

__author__ = """Elias Dabbas"""
__email__ = 'eliasdabbas@gmail.com'
__version__ = '0.4.1'

from advertools.ad_create import ad_create
from advertools.ad_from_string import ad_from_string
from advertools.extract import extract_mentions, extract_hashtags, extract_emoji
from advertools.kw_generate import *
from advertools.stopwords import stopwords
from advertools.url_builders import url_utm_ga
from advertools.word_frequency import word_frequency
from . import twitter
from .serp import serp_goog, VALID_VALUES, set_logging_level



import unittest
from itertools import permutations, combinations

from advertools.kw_generate import *

import pandas as pd


class KeywordTests(unittest.TestCase):

    def test_match_types_are_capitalized(self):
        df = kw_generate(['one', 'two'], ['three', 'four'],
                         match_types=['eXact', 'PHRase', 'BRoad'])
        self.assertEqual(set(df['Criterion Type']),
                         {'Exact', 'Phrase', 'Broad'})

    def test_raises_error_for_wrong_match_types(self):
        with self.assertRaises(ValueError):
            kw_generate(['one', 'two'], ['three', 'four'],
                        match_types=['hello', 'world'])

    def test_raises_error_for_wrong_max_len(self):
        with self.assertRaises(ValueError):
            kw_generate(['one', 'two'], ['three', 'four'], max_len=1)

    def test_right_combinatorics_fuction_selected(self):
        df_perm = kw_generate(['one', 'two'], ['three', 'four'],
                              order_matters=True)
        self.assertTrue({'one three', 'three one'}
                        .issubset(set(df_perm['Keyword'])))
        df_comb = kw_generate(['one', 'two'], ['three', 'four'],
                              order_matters=False)
        self.assertFalse({'one three', 'three one'}
                         .issubset(set(df_comb['Keyword'])))

    def test_correct_column_names_used(self):
        headers = ['Campaign', 'Ad Group', 'Keyword',
                   'Criterion Type', 'Labels']
        df = kw_generate(['one', 'two'], ['three', 'four'])
        self.assertEqual(list(df.columns.values), headers)

    def test_all_products_in_final_result(self):
        products = {'One', 'Two', 'Three'}
        df = kw_generate(products, ['four', 'five'])
        self.assertEqual(products, set(df['Ad Group']))

    def test_keyword_lengths_vary_between_2_max_len(self):
        df2 = kw_generate(['one', 'two'], ['three', 'four'], max_len=2)
        self.assertEqual(set(df2['Keyword'].str.count(' ')+1), {2})
        df3 = kw_generate(['one', 'two'], ['three', 'four'], max_len=3)
        self.assertEqual(set(df3['Keyword'].str.count(' ')+1), {2, 3})
        df4 = kw_generate(['one', 'two'], ['three', 'four', 'five'], max_len=4)
        self.assertEqual(set(df4['Keyword'].str.count(' ')+1), {2, 3, 4})

    def test_correct_combinations_are_generated(self):
        products = ['one', 'two']
        words = ['three', 'four']
        test_combs = []
        for i in range(2, 4):
            for prod in products:
                for comb in combinations([prod] + words, i):
                    if prod not in comb:
                        continue
                    test_combs.append(' '.join(comb))

        df = kw_generate(products, words, match_types=['Exact'],
                         order_matters=False)
        self.assertEqual(set(test_combs), set(df['Keyword']))

    def test_correct_permutations_are_generated(self):
        products = ['one', 'two']
        words = ['three', 'four']
        test_combs = []
        for i in range(2, 4):
            for prod in products:
                for perm in permutations([prod] + words, i):
                    if prod not in perm:
                        continue
                    test_combs.append(' '.join(perm))

        df = kw_generate(products, words, match_types=['Exact'],
                         order_matters=True)
        self.assertEqual(set(test_combs), set(df['Keyword']))

    def test_correct_campaign_name(self):
        df = kw_generate(['one', 'two'], ['three'], campaign_name='My_campaign')
        self.assertEqual(df['Campaign'][0], 'My_campaign')
        self.assertTrue(len(set(df['Campaign'])) == 1)

    def test_ad_group_names_capitalized(self):
        df = kw_generate(['oNe', 'TWO', 'THree', 'four', 'fivE'], ['six'])
        self.assertEqual(set(['One', 'Two', 'Three', 'Four', 'Five']),
                         set(df['Ad Group']))

    def test_modified_adds_plus_sign_as_many_as_spaces(self):
        df = kw_generate(['one', 'two'], ['three'], match_types=['Modified'])
        self.assertTrue(all(df['Keyword'].str.count('\+')
                            == df['Keyword'].str.count(' ')+1))
        self.assertTrue(set(df['Criterion Type']) == {'Broad'})

    def test_all_match_types_used(self):
        df_exact = kw_generate(['one'], ['two'], match_types=['Exact'])
        df_phrase = kw_generate(['one'], ['two'], match_types=['Phrase'])
        df_broad = kw_generate(['one'], ['two'], match_types=['Broad'])
        df_e_p_b = kw_generate(['one'], ['two'],
                               match_types=['Exact', 'Phrase', 'Broad',
                                            'Modified'])
        self.assertEqual(set(df_exact['Criterion Type']), {'Exact'})
        self.assertEqual(set(df_phrase['Criterion Type']), {'Phrase'})
        self.assertEqual(set(df_broad['Criterion Type']), {'Broad'})
        self.assertEqual(set(df_e_p_b['Criterion Type']),
                         {'Exact', 'Phrase', 'Broad'})

    def test_returns_pd_dataframe(self):
        df = kw_generate(['one'], ['two'], order_matters=True)
        self.assertEqual(type(df), pd.core.frame.DataFrame)
        self.assertEqual(df.shape, (6, 5))


words = ['[one]', '"two"', '+three -four', '[five', 'six"',
         '+seven eight nine]']


def test_kw_broad():
    result = kw_broad(words)
    assert result == ['one', 'two', 'three -four', 'five', 'six',
                      'seven eight nine']


def test_kw_exact():
    result = kw_exact(words)
    assert result == ['[one]', '[two]', '[three -four]', '[five]', '[six]',
                      '[seven eight nine]']


def test_exact_contains_brackets():
    result = kw_exact(words)
    assert ['[' in x and ']' in x for x in result]


def test_kw_phrase():
    result = kw_phrase(words)
    assert result == ['"one"', '"two"', '"three -four"', '"five"', '"six"',
                      '"seven eight nine"']


def test_kw_modified():
    result = kw_modified(words)
    assert result == ['+one', '+two', '+three +-four', '+five', '+six',
                      '+seven +eight +nine']


def test_kw_neg_broad():
    result = kw_neg_broad(words)
    assert result == ['-one', '-two', '-three -four', '-five', '-six',
                      '-seven eight nine']


def test_kw_neg_exact():
    result = kw_neg_exact(words)
    assert result == ['-[one]', '-[two]', '-[three -four]', '-[five]', '-[six]',
                      '-[seven eight nine]']


def test_kw_neg_phrase():
    result = kw_neg_phrase(words)
    assert result == ['-"one"', '-"two"', '-"three -four"', '-"five"', '-"six"',
                      '-"seven eight nine"']


if __name__ == '__main__':
        unittest.main()




import re
from collections import Counter
from .emoji_dict import emoji_dict
from .emoji_dict import emoji_regexp as EMOJI

HASHTAG = r'#\w+'
MENTION = r'(?:^|\W)([@＠][A-Za-z0-9_]+)'


def extract_mentions(text_list):
    """Return a summary dictionary about mentions in ``text_list``

    Get a summary of the number of mentions, their frequency, the top
    ones, and more.

    :param text_list: A list of text strings.
    :returns summary: A dictionary with various stats about mentions

    >>> posts = ['hello @john and @jenny', 'hi there @john', 'good morning']
    >>> mention_summary = extract_mentions(posts)
    >>> mention_summary.keys()
    dict_keys(['mentions', 'mentions_flat', 'mention_counts', 'mention_freq',
    'top_mentions', 'overview'])

    >>> mention_summary['mentions']
    [['@john', '@jenny'], ['@john'], []]

    A simple extract of mentions from each of the posts. An empty list if
    none exist

    >>> mention_summary['mentions_flat']
    ['@john', '@jenny', '@john']

    All mentions in one flat list.

    >>> mention_summary['mention_counts']
    [2, 1, 0]

    The count of mentions per post.

    >>> mention_summary['mention_freq']
    [(0, 1), (1, 1), (2, 1)]

    Shows how many posts had 0, 1, 2, 3, etc. mentions
    (number_of_mentions, count)

    >>> mention_summary['top_mentions']
    [('@john', 2), ('@jenny', 1)]

    >>> mention_summary['overview']
    {'num_posts': 3, # number of posts
     'num_mentions': 3,
     'mentions_per_post': 1.0,
     'unique_mentions': 2}
    """
    mentions = [re.findall(MENTION, text.lower()) for text in text_list]
    mentions_flat = [item for sublist in mentions for item in sublist]
    summary = {
        'mentions': mentions,
        'mentions_flat': mentions_flat,
        'mention_counts': [len(mention) for mention in mentions],
        'mention_freq': sorted(Counter([len(mention)
                                        for mention in mentions]).items(),
                               key=lambda x: x[0]),
        'top_mentions': sorted(Counter(mentions_flat).items(),
                               key=lambda x: x[1],
                               reverse=True),
        'overview': {
            'num_posts': len(text_list),
            'num_mentions': len(mentions_flat),
            'mentions_per_post': len(mentions_flat) / len(text_list),
            'unique_mentions': len(set(mentions_flat)),
        }
    }
    return summary


def extract_hashtags(text_list):
    """Return a summary dictionary about hashtags in ``text_list``

    Get a summary of the number of hashtags, their frequency, the top
    ones, and more.

    :param text_list: A list of text strings.
    :returns summary: A dictionary with various stats about hashtags

    >>> posts = ['i like #blue', 'i like #green and #blue', 'i like all']
    >>> hashtag_summary = extract_hashtags(posts)
    >>> hashtag_summary.keys()
    dict_keys(['hashtags', 'hashtags_flat', 'hashtag_counts', 'hashtag_freq',
    'top_hashtags', 'overview'])

    >>> hashtag_summary['hashtags']
    [['#blue'], ['#green', '#blue'], []]

    A simple extract of hashtags from each of the posts. An empty list if
    none exist

    >>> hashtag_summary['hashtags_flat']
    ['#blue', '#green', '#blue']

    All hashtags in one flat list.

    >>> hashtag_summary['hashtag_counts']
    [1, 2, 0]

    The count of hashtags per post.

    >>> hashtag_summary['hashtag_freq']
    [(0, 1), (1, 1), (2, 1)]

    Shows how many posts had 0, 1, 2, 3, etc. hashtags
    (number_of_hashtags, count)

    >>> hashtag_summary['top_hashtags']
    [('#blue', 2), ('#green', 1)]

    >>> hashtag_summary['overview']
    {'num_posts': 3,
     'num_hashtags': 3,
     'hashtags_per_post': 1.0,
     'unique_hashtags': 2}
     """
    hashtags = [re.findall(HASHTAG, text.lower()) for text in text_list]
    hashtags_flat = [item for sublist in hashtags for item in sublist]
    summary = {
        'hashtags': hashtags,
        'hashtags_flat': hashtags_flat,
        'hashtag_counts': [len(hashtag) for hashtag in hashtags],
        'hashtag_freq': sorted(Counter([len(hashtag)
                                        for hashtag in hashtags]).items(),
                               key=lambda x: x[0]),
        'top_hashtags': sorted(Counter(hashtags_flat).items(),
                               key=lambda x: x[1],
                               reverse=True),
        'overview': {
            'num_posts': len(text_list),
            'num_hashtags': len(hashtags_flat),
            'hashtags_per_post': len(hashtags_flat) / len(text_list),
            'unique_hashtags': len(set(hashtags_flat)),
        }
    }
    return summary


def extract_emoji(text_list):
    """Return a summary dictionary about emoji in ``text_list``

    Get a summary of the number of emoji, their frequency, the top
    ones, and more.

    :param text_list: A list of text strings.
    :returns summary: A dictionary with various stats about emoji

    >>> posts = ['I am grinning 😀','A grinning cat 😺',
                 'hello! 😀😀😀 💛💛', 'Just text']

    >>> emoji_summary = extract_emoji(posts)
    >>> emoji_summary.keys()
    dict_keys(['emoji', 'emoji_text', 'emoji_flat', 'emoji_flat_text',
               'emoji_counts', 'emoji_freq', 'top_emoji',
               'top_emoji_text', 'overview'])

    >>> emoji_summary['emoji']
    [['😀'], ['😺'], ['😀', '😀', '😀', '💛', '💛'], []]

    >>> emoji_summary['emoji_text']
    [['grinning face'],
     ['grinning cat face'],
     ['grinning face', 'grinning face', 'grinning face',
      'yellow heart', 'yellow heart'],
     []]

    A simple extract of emoji from each of the posts. An empty
    list if none exist

    >>> emoji_summary['emoji_flat']
    ['😀', '😺', '😀', '😀', '😀', '💛', '💛']

    >>> emoji_summary['emoji_flat_text']
    ['grinning face', 'grinning cat face', 'grinning face', 'grinning face',
     'grinning face', 'yellow heart', 'yellow heart']


    All emoji in one flat list.

    >>> emoji_summary['emoji_counts']
    [1, 1, 5, 0]

    The count of emoji per post.

    >>> emoji_summary['emoji_freq']
    [(0, 1), (1, 2), (5, 1)]

    Shows how many posts had 0, 1, 2, 3, etc. emoji
    (number_of_emoji, count)

    >>> emoji_summary['top_emoji']
    [('😀', 4), ('💛', 2), ('😺', 1)]

    >>> emoji_summary['top_emoji_text']
    [('grinning face', 4), ('yellow heart', 2),
     ('grinning cat face', 1)]

    >>> emoji_summary['overview']
    {'num_posts': 4,
     'num_emoji': 7,
     'emoji_per_post': 1.75,
     'unique_emoji': 3}
    """
    emoji = [re.findall(EMOJI, text.lower()) for text in text_list]
    emoji_flat = [item for sublist in emoji for item in sublist]
    emoji_flat_text = [emoji_dict[em].strip(':').replace('_', ' ')
                       for em in emoji_flat]
    summary = {
        'emoji': emoji,
        'emoji_text': [[emoji_dict[em].strip(':').replace('_', ' ')
                        for em in em_list]
                       for em_list in emoji],
        'emoji_flat': emoji_flat,
        'emoji_flat_text': emoji_flat_text,
        'emoji_counts': [len(em) for em in emoji],
        'emoji_freq': sorted(Counter([len(em) for em in emoji]).items(),
                             key=lambda x: x[0]),
        'top_emoji': sorted(Counter(emoji_flat).items(),
                            key=lambda x: x[1],
                            reverse=True),
        'top_emoji_text': sorted(Counter(emoji_flat_text).items(),
                                 key=lambda x: x[1],
                                 reverse=True),
        'overview': {
            'num_posts': len(text_list),
            'num_emoji': len(emoji_flat),
            'emoji_per_post': len(emoji_flat) / len(text_list),
            'unique_emoji': len(set(emoji_flat)),
        }

    }
    return summary



import datetime
import logging
from itertools import product

import pandas as pd
from pandas.io.json import json_normalize
import requests

LOG_FMT = '%(asctime)s | %(levelname)s | %(filename)s:%(lineno)d | %(funcName)s | %(message)s'
logging.basicConfig(format=LOG_FMT)


VALID_VALUES = dict(
    fileType={
        'bas', 'c', 'cc', 'cpp', 'cs', 'cxx', 'doc', 'docx', 'dwf', 'gpx',
        'h', 'hpp', 'htm', 'html', 'hwp', 'java', 'kml', 'kmz', 'odp', 'ods',
        'odt', 'pdf', 'pl', 'ppt', 'pptx', 'ps', 'py', 'rtf', 'svg', 'swf',
        'tex', 'text', 'txt', 'wap', 'wml', 'xls', 'xlsx', 'xml',
    },

    c2coff={0, 1},

    cr={
        'countryAF', 'countryAL', 'countryDZ', 'countryAS', 'countryAD',
        'countryAO', 'countryAI', 'countryAQ', 'countryAG', 'countryAR',
        'countryAM', 'countryAW', 'countryAU', 'countryAT', 'countryAZ',
        'countryBS', 'countryBH', 'countryBD', 'countryBB', 'countryBY',
        'countryBE', 'countryBZ', 'countryBJ', 'countryBM', 'countryBT',
        'countryBO', 'countryBA', 'countryBW', 'countryBV', 'countryBR',
        'countryIO', 'countryBN', 'countryBG', 'countryBF', 'countryBI',
        'countryKH', 'countryCM', 'countryCA', 'countryCV', 'countryKY',
        'countryCF', 'countryTD', 'countryCL', 'countryCN', 'countryCX',
        'countryCC', 'countryCO', 'countryKM', 'countryCG', 'countryCD',
        'countryCK', 'countryCR', 'countryCI', 'countryHR', 'countryCU',
        'countryCY', 'countryCZ', 'countryDK', 'countryDJ', 'countryDM',
        'countryDO', 'countryTP', 'countryEC', 'countryEG', 'countrySV',
        'countryGQ', 'countryER', 'countryEE', 'countryET', 'countryEU',
        'countryFK', 'countryFO', 'countryFJ', 'countryFI', 'countryFR',
        'countryFX', 'countryGF', 'countryPF', 'countryTF', 'countryGA',
        'countryGM', 'countryGE', 'countryDE', 'countryGH', 'countryGI',
        'countryGR', 'countryGL', 'countryGD', 'countryGP', 'countryGU',
        'countryGT', 'countryGN', 'countryGW', 'countryGY', 'countryHT',
        'countryHM', 'countryVA', 'countryHN', 'countryHK', 'countryHU',
        'countryIS', 'countryIN', 'countryID', 'countryIR', 'countryIQ',
        'countryIE', 'countryIL', 'countryIT', 'countryJM', 'countryJP',
        'countryJO', 'countryKZ', 'countryKE', 'countryKI', 'countryKP',
        'countryKR', 'countryKW', 'countryKG', 'countryLA', 'countryLV',
        'countryLB', 'countryLS', 'countryLR', 'countryLY', 'countryLI',
        'countryLT', 'countryLU', 'countryMO', 'countryMK', 'countryMG',
        'countryMW', 'countryMY', 'countryMV', 'countryML', 'countryMT',
        'countryMH', 'countryMQ', 'countryMR', 'countryMU', 'countryYT',
        'countryMX', 'countryFM', 'countryMD', 'countryMC', 'countryMN',
        'countryMS', 'countryMA', 'countryMZ', 'countryMM', 'countryNA',
        'countryNR', 'countryNP', 'countryNL', 'countryAN', 'countryNC',
        'countryNZ', 'countryNI', 'countryNE', 'countryNG', 'countryNU',
        'countryNF', 'countryMP', 'countryNO', 'countryOM', 'countryPK',
        'countryPW', 'countryPS', 'countryPA', 'countryPG', 'countryPY',
        'countryPE', 'countryPH', 'countryPN', 'countryPL', 'countryPT',
        'countryPR', 'countryQA', 'countryRE', 'countryRO', 'countryRU',
        'countryRW', 'countrySH', 'countryKN', 'countryLC', 'countryPM',
        'countryVC', 'countryWS', 'countrySM', 'countryST', 'countrySA',
        'countrySN', 'countryCS', 'countrySC', 'countrySL', 'countrySG',
        'countrySK', 'countrySI', 'countrySB', 'countrySO', 'countryZA',
        'countryGS', 'countryES', 'countryLK', 'countrySD', 'countrySR',
        'countrySJ', 'countrySZ', 'countrySE', 'countryCH', 'countrySY',
        'countryTW', 'countryTJ', 'countryTZ', 'countryTH', 'countryTG',
        'countryTK', 'countryTO', 'countryTT', 'countryTN', 'countryTR',
        'countryTM', 'countryTC', 'countryTV', 'countryUG', 'countryUA',
        'countryAE', 'countryUK', 'countryUS', 'countryUM', 'countryUY',
        'countryUZ', 'countryVU', 'countryVE', 'countryVN', 'countryVG',
        'countryVI', 'countryWF', 'countryEH', 'countryYE', 'countryYU',
        'countryZM', 'countryZW'
    },

    gl={
        'ad', 'ae', 'af', 'ag', 'ai', 'al', 'am', 'an', 'ao', 'aq', 'ar',
        'as', 'at', 'au', 'aw', 'az', 'ba', 'bb', 'bd', 'be', 'bf', 'bg',
        'bh', 'bi', 'bj', 'bm', 'bn', 'bo', 'br', 'bs', 'bt', 'bv', 'bw',
        'by', 'bz', 'ca', 'cc', 'cd', 'cf', 'cg', 'ch', 'ci', 'ck', 'cl',
        'cm', 'cn', 'co', 'cr', 'cs', 'cu', 'cv', 'cx', 'cy', 'cz', 'de',
        'dj', 'dk', 'dm', 'do', 'dz', 'ec', 'ee', 'eg', 'eh', 'er', 'es',
        'et', 'fi', 'fj', 'fk', 'fm', 'fo', 'fr', 'ga', 'gd', 'ge', 'gf',
        'gh', 'gi', 'gl', 'gm', 'gn', 'gp', 'gq', 'gr', 'gs', 'gt', 'gu',
        'gw', 'gy', 'hk', 'hm', 'hn', 'hr', 'ht', 'hu', 'id', 'ie', 'il',
        'in', 'io', 'iq', 'ir', 'is', 'it', 'jm', 'jo', 'jp', 'ke', 'kg',
        'kh', 'ki', 'km', 'kn', 'kp', 'kr', 'kw', 'ky', 'kz', 'la', 'lb',
        'lc', 'li', 'lk', 'lr', 'ls', 'lt', 'lu', 'lv', 'ly', 'ma', 'mc',
        'md', 'mg', 'mh', 'mk', 'ml', 'mm', 'mn', 'mo', 'mp', 'mq', 'mr',
        'ms', 'mt', 'mu', 'mv', 'mw', 'mx', 'my', 'mz', 'na', 'nc', 'ne',
        'nf', 'ng', 'ni', 'nl', 'no', 'np', 'nr', 'nu', 'nz', 'om', 'pa',
        'pe', 'pf', 'pg', 'ph', 'pk', 'pl', 'pm', 'pn', 'pr', 'ps', 'pt',
        'pw', 'py', 'qa', 're', 'ro', 'ru', 'rw', 'sa', 'sb', 'sc', 'sd',
        'se', 'sg', 'sh', 'si', 'sj', 'sk', 'sl', 'sm', 'sn', 'so', 'sr',
        'st', 'sv', 'sy', 'sz', 'tc', 'td', 'tf', 'tg', 'th', 'tj', 'tk',
        'tl', 'tm', 'tn', 'to', 'tr', 'tt', 'tv', 'tw', 'tz', 'ua', 'ug',
        'uk', 'um', 'us', 'uy', 'uz', 'va', 'vc', 've', 'vg', 'vi', 'vn',
        'vu', 'wf', 'ws', 'ye', 'yt', 'za', 'zm', 'zw',
    },

    filter={0, 1},

    hl={
        'af', 'sq', 'sm', 'ar', 'az', 'eu', 'be', 'bn', 'bh', 'bs', 'bg',
        'ca', 'zh-CN', 'zh-TW', 'hr', 'cs', 'da', 'nl', 'en', 'eo', 'et',
        'fo', 'fi', 'fr', 'fy', 'gl', 'ka', 'de', 'el', 'gu', 'iw', 'hi',
        'hu', 'is', 'id', 'ia', 'ga', 'it', 'ja', 'jw', 'kn', 'ko', 'la',
        'lv', 'lt', 'mk', 'ms', 'ml', 'mt', 'mr', 'ne', 'no', 'nn', 'oc',
        'fa', 'pl', 'pt-BR', 'pt-PT', 'pa', 'ro', 'ru', 'gd', 'sr', 'si',
        'sk', 'sl', 'es', 'su', 'sw', 'sv', 'tl', 'ta', 'te', 'th', 'ti',
        'tr', 'uk', 'ur', 'uz', 'vi', 'cy', 'xh', 'zu'
    },

    imgColorType={
        'color', 'gray', 'mono'
    },

    imgDominantColor={
        'black',
        'blue',
        'brown',
        'gray',
        'green',
        'orange',
        'pink',
        'purple',
        'red',
        'teal',
        'white',
        'yellow',
    },

    imgSize={
        'huge',
        'icon',
        'large',
        'medium',
        'small',
        'xlarge',
        'xxlarge',
    },

    imgType={
        'clipart',
        'face',
        'lineart',
        'news',
        'photo',
    },

    lr={
        'lang_ar', 'lang_bg', 'lang_ca', 'lang_zh-CN', 'lang_zh-TW',
        'lang_hr', 'lang_cs', 'lang_da', 'lang_nl', 'lang_en', 'lang_et',
        'lang_fi', 'lang_fr', 'lang_de', 'lang_el', 'lang_iw', 'lang_hu',
        'lang_is', 'lang_id', 'lang_it', 'lang_ja', 'lang_ko', 'lang_lv',
        'lang_lt', 'lang_no', 'lang_pl', 'lang_pt', 'lang_ro', 'lang_ru',
        'lang_sr', 'lang_sk', 'lang_sl', 'lang_es', 'lang_sv', 'lang_tr',
    },

    num={1, 2, 3, 4, 5, 6, 7, 8, 9, 10},

    rights={
        'cc_publicdomain', 'cc_attribute', 'cc_sharealike',
        'cc_noncommercial', 'cc_nonderived'
    },

    safe={'active', 'off'},

    searchType={None, 'image'},

    siteSearchFilter={'e', 'i'},

    start=range(1, 92)
)


def _dict_product(d):
    """Return the product of all values of a dict, while
    coupling each value with its key.
    This is used to generate multiple queries out of
        possibly multiple arguments in serp_goog.

    >>> d = {'a': [1], 'b': [2, 3, 4], 'c': [5, 6]}
    >>> _dict_product(d)
    >>> [{'a': 1, 'b': 2, 'c': 5},
         {'a': 1, 'b': 2, 'c': 6},
         {'a': 1, 'b': 3, 'c': 5},
         {'a': 1, 'b': 3, 'c': 6},
         {'a': 1, 'b': 4, 'c': 5},
         {'a': 1, 'b': 4, 'c': 6}]
    """
    dicts = []
    for prod in product(*d.values()):
        tempdict = dict(zip(d.keys(), prod))
        dicts.append(tempdict)
    return dicts


def serp_goog(q, cx, key, c2coff=None, cr=None,
              dateRestrict=None, exactTerms=None, excludeTerms=None,
              fileType=None, filter=None, gl=None, highRange=None,
              hl=None, hq=None, imgColorType=None, imgDominantColor=None,
              imgSize=None, imgType=None, linkSite=None, lowRange=None,
              lr=None, num=None, orTerms=None, relatedSite=None,
              rights=None, safe=None, searchType=None, siteSearch=None,
              siteSearchFilter=None, sort=None, start=None):
    """Query Google and get search results in a DataFrame.

    For each parameter, you can supply single or multiple values / arguments.
        If you pass multiple arguments, all the possible combinations of
        arguments (the product) will be requested, and you will get one
        DataFrame combining all queries. See examples below.

    :param q: The search expression.
    :param cx: The custom search engine ID to use for this
        request.
    :param key: The API key of your custom search engine.
    :param c2coff: Enables or disables Simplified and
        Traditional Chinese Search. The default value for this
        parameter is 0 (zero), meaning that the feature is enabled.
        Supported values are:1: Disabled0: Enabled (default)
    :param cr: Restricts search results to documents
        originating in a particular country. You may use Boolean
        operators in the cr parameter's value.Google Search
        determines the country of a document by analyzing:the top-
        level domain (TLD) of the document's URLthe geographic
        location of the Web server's IP addressSee the Country
        Parameter Values page for a list of valid values for this
        parameter.
    :param dateRestrict: Restricts results to URLs based on
        date. Supported values include:d[number]: requests results
        from the specified number of past days.
        - w[number]: requests results from the specified number of past weeks.
        - m[number]: requests results from the specified number of past months.
        - y[number]: requests results from the specified number of past years.
    :param exactTerms: Identifies a phrase that all
        documents in the search results must contain.
    :param excludeTerms: Identifies a word or phrase that
        should not appear in any documents in the search results.
    :param fileType: Restricts results to files of a
        specified extension. A list of file types indexable by
        Google can be found in Search Console Help Center.
    :param filter: Controls turning on or off the duplicate
        content filter.See Automatic Filtering for more information
        about Google's search results filters. Note that host
        crowding filtering applies only to multi-site searches.By
        default, Google applies filtering to all search results to
        improve the quality of those results.  Acceptable values
        are:  "0": Turns off duplicate content filter.  "1": Turns
        on duplicate content filter.
    :param gl: Geolocation of end user. The gl parameter
        value is a two-letter country code. The gl parameter boosts
        search results whose country of origin matches the parameter
        value. See the Country Codes page for a list of valid
        values.Specifying a gl parameter value should lead to more
        relevant results. This is particularly true for
        international customers and, even more specifically, for
        customers in English- speaking countries other than the
        United States.
    :param highRange: Specifies the ending value for a
        search range.Use lowRange and highRange to append an
        inclusive search range of lowRange...highRange to the query.
    :param hl: Sets the user interface language. Explicitly
        setting this parameter improves the performance and the
        quality of your search results.See the Interface
        Languages section of Internationalizing Queries and Results
        Presentation for more information, and Supported Interface
        Languages for a list of supported languages.
    :param hq: Appends the specified query terms to the
        query, as if they were combined with a logical AND operator.
    :param imgColorType: Returns black and white, grayscale,
        or color images: mono, gray, and color.  Acceptable values
        are:  "color": color  "gray": gray  "mono": mono
    :param imgDominantColor: Returns images of a specific
        dominant color.  Acceptable values are:  "black": black
        "blue": blue  "brown": brown  "gray": gray  "green": green
        "orange": orange  "pink": pink  "purple": purple  "red": red
        "teal": teal  "white": white  "yellow": yellow
    :param imgSize: Returns images of a specified size.
        Acceptable values are:  "huge": huge  "icon": icon  "large":
        large  "medium": medium  "small": small  "xlarge": xlarge
        "xxlarge": xxlarge
    :param imgType: Returns images of a type.  Acceptable
        values are:  "clipart": clipart  "face": face  "lineart":
        lineart  "news": news  "photo": photo
    :param linkSite: Specifies that all search results
        should contain a link to a particular URL
    :param lowRange: Specifies the starting value for a
        search range. Use lowRange and highRange to append an
        inclusive search range of lowRange...highRange to the query.
    :param lr: Restricts the search to documents written in
        a particular language (e.g., lr=lang_ja).  Acceptable values
        are:  "lang_ar": Arabic  "lang_bg": Bulgarian  "lang_ca":
        Catalan  "lang_cs": Czech  "lang_da": Danish  "lang_de":
        German  "lang_el": Greek  "lang_en": English  "lang_es":
        Spanish  "lang_et": Estonian  "lang_fi": Finnish  "lang_fr":
        French  "lang_hr": Croatian  "lang_hu": Hungarian
        "lang_id": Indonesian  "lang_is": Icelandic  "lang_it":
        Italian  "lang_iw": Hebrew  "lang_ja": Japanese  "lang_ko":
        Korean  "lang_lt": Lithuanian  "lang_lv": Latvian
        "lang_nl": Dutch  "lang_no": Norwegian  "lang_pl": Polish
        "lang_pt": Portuguese  "lang_ro": Romanian  "lang_ru":
        Russian  "lang_sk": Slovak  "lang_sl": Slovenian  "lang_sr":
        Serbian  "lang_sv": Swedish  "lang_tr": Turkish  "lang_zh-
        CN": Chinese (Simplified)  "lang_zh-TW": Chinese
        (Traditional)
    :param num: Number of search results to return.Valid
        values are integers between 1 and 10, inclusive.
    :param orTerms: Provides additional search terms to
        check for in a document, where each document in the search
        results must contain at least one of the additional search
        terms.
    :param relatedSite: Specifies that all search results
        should be pages that are related to the specified URL.
    :param rights: Filters based on licensing. Supported
        values include: cc_publicdomain, cc_attribute,
        cc_sharealike, cc_noncommercial, cc_nonderived, and
        combinations of these.
    :param safe: Search safety level.  Acceptable values
        are:  "active": Enables SafeSearch filtering.  "off":
        Disables SafeSearch filtering.  (default)
    :param searchType: Specifies the search type: image. If
        unspecified, results are limited to webpages.  Acceptable
        values are:  "image": custom image search.
    :param siteSearch: Specifies all search results should
        be pages from a given site.
    :param siteSearchFilter: Controls whether to include or
        exclude results from the site named in the siteSearch
        parameter.  Acceptable values are:  "e": exclude  "i":
        include
    :param sort: The sort expression to apply to the
        results.
    :param start: The index of the first result to
        return.Valid value are integers starting 1 (default) and the
        second result is 2 and so forth. For example &start=11 gives
        the second page of results with the default "num" value of
        10 results per page.Note: No more than 100 results will ever
        be returned for any query with JSON API, even if more than
        100 documents match the query, so setting (start + num) to
        more than 100 will produce an error. Note that the maximum
        value for num is 10.

    The following function call will produce two queries:
    "hotel" in English, and "hotel"

    >>> serp_goog(q='hotel', gl=['en', 'fr'])

    The below function call will prouce four queries and make four requests:
    "fligts" in UK
    "fligts" in Australia
    "tickets" in UK
    "tickets" in Australia

    >>> serp_goog(q=['flights', 'tickets'], cr=['uk', 'au'])
    """
    params = locals()
    supplied_params = {k: v for k, v in params.items() if params[k]}

    for p in supplied_params:
        if isinstance(supplied_params[p], (str, int)):
            supplied_params[p] = [supplied_params[p]]

    for p in supplied_params:
        if p in VALID_VALUES:
            if not set(supplied_params[p]).issubset(VALID_VALUES[p]):
                raise ValueError('Please make sure you provide a'
                                 ' valid value for "{}", valid values:\n'
                                 '{}'.format(p, sorted(VALID_VALUES[p])))
    params_list = _dict_product(supplied_params)
    base_url = 'https://www.googleapis.com/customsearch/v1?'
    ordered_cols = ['searchTerms', 'rank', 'title', 'snippet',
                    'displayLink', 'link', 'queryTime', 'totalResults']
    responses = []
    for param in params_list:
        param_log = ', '.join([k + '=' + str(v) for k, v in param.items()])
        logging.info(msg='Requesting: ' + param_log)
        resp = requests.get(base_url, params=param)
        resp.raise_for_status()
        responses.append(resp)

    result_df = pd.DataFrame()
    for resp in responses:
        request_metadata = resp.json()['queries']['request'][0]
        del request_metadata['title']
        search_info = resp.json()['searchInformation']
        if int(search_info['totalResults']) == 0:
            df = pd.DataFrame(columns=ordered_cols, index=range(1))
            df['searchTerms'] = request_metadata['searchTerms']
        else:
            df = pd.DataFrame(resp.json()['items'])
            df['cseName'] = resp.json()['context']['title']
            start_idx = request_metadata['startIndex']
            df['rank'] = range(start_idx, start_idx + len(df))

        meta_columns = {**request_metadata, **search_info}
        df = df.assign(**meta_columns)
        df['queryTime'] = datetime.datetime.now(tz=datetime.timezone.utc)
        df['queryTime'] = pd.to_datetime(df['queryTime'])
        if 'image' in df:
            img_df = json_normalize(df['image'])
            img_df.columns = ['image.' + c for c in img_df.columns]
            df = pd.concat([df, img_df], axis=1)
        result_df = result_df.append(df, sort=False, ignore_index=True)
    non_ordered = [c for c in result_df.columns if c not in ordered_cols]
    result_df = result_df[ordered_cols + non_ordered]
    return result_df


def set_logging_level(level_or_name):
    """Change the logging level during the session.
    Acceptable values are [0, 10, 20, 30, 40, 50,
    'NOTSET', 'DEBUG', 'INFO', 'WARNING', 'ERROR',
    'CRITICAL']
    """
    lvl_names_values = [0, 10, 20, 30, 40, 50,
                        'NOTSET', 'DEBUG', 'INFO',
                        'WARNING', 'ERROR', 'CRITICAL']
    if level_or_name not in lvl_names_values:
        raise ValueError('Please make sure you supply'
                         ' a value from: {}'.format(lvl_names_values))
    logging.getLogger().setLevel(level_or_name)


logging.getLogger().setLevel('INFO')



import logging
import os
from itertools import product

import pandas as pd
import pytest

from advertools.serp import (serp_goog, VALID_VALUES,
                             _dict_product, set_logging_level)

goog_cse_cx = os.environ.get('GOOG_CSE_CX')
goog_cse_key = os.environ.get('GOOG_CSE_KEY')


def test_dict_product_produces_correct_result():
    d = {'a': [1, 2, 3], 'b': [4, 5, 6], 'c': [10, 20]}
    dp = _dict_product(d)
    assert (list(product(*d.values())) ==
            [tuple(x.values()) for x in dp])
    assert ([tuple(x.keys()) for x in dp] ==
            [tuple(d.keys()) for x in range(len(dp))])


def test_dict_product_return_correct_types():
    d = {'a': [1], 'b': [10, 20, 30], 'c': (4, 5, 6)}
    dp = _dict_product(d)
    assert isinstance(dp, list)
    assert [isinstance(x, dict) for x in dp]
    assert len(dp) == len(list(product(*d.values())))


def test_serp_goog_raises_error_on_invalid_args():
    with pytest.raises(ValueError):
        for val in VALID_VALUES:
            params = {val: 'WRONG VALUE'}
            serp_goog(q='q', cx='cx', key='key', **params)


def test_serp_goog_return_correct_result():
    result = serp_goog(q='testing hotels', cx=goog_cse_cx,
                       key=goog_cse_key, searchType=['image', None])
    assert isinstance(result, pd.core.frame.DataFrame)
    assert 'title' in result
    assert 'image' in result
    assert len(result) == 20


def test_serp_goog_handles_no_search_results():
    q = 'aquerythatdoesntgetrezultssss'
    result = serp_goog(q=q, cx=goog_cse_cx, key=goog_cse_key,
                       cr='countryRU', hl='zh-TW', gl='nf')
    assert len(result) == 1
    assert result['searchTerms'].values[0] == q


def test_correctly_changing_log_levels():
    lvl_names_values = [0, 10, 20, 30, 40, 50]
    for level in lvl_names_values:
        set_logging_level(level)
        assert logging.getLogger().level == level
    with pytest.raises(ValueError):
        set_logging_level('WRONG VALUE')
